@@ -199,7 +199,16 @@ typedef struct Avision_HWEntry {
   /*second enum cause 32 bit int above is full*/
   enum {
     /* force no calibration */
-    AV_NO_TUNE_SCAN_LENGTH = (1<<0)
+    AV_NO_TUNE_SCAN_LENGTH = (1<<0),
+
+    /* for gray scans, set grey filter */
+    AV_USE_GRAY_FILTER = (1<<1),
+
+    /* For (HP) scanners with flipping duplexers */
+    AV_ADF_FLIPPING_DUPLEX = (1<<2),
+
+    /* For scanners which need to have their firmware read to properly function. */
+    AV_FIRMWARE = (1<<3)
   } feature_type2;
 
 } Avision_HWEntry;
@@ -282,6 +291,7 @@ enum Avision_Option
   OPT_IR,                /* infra-red */
   OPT_MULTISAMPLE,       /* multi-sample */
   
+  OPT_MISC_GROUP,
   OPT_FRAME,             /* Film holder control */
 
   OPT_POWER_SAVE_TIME,   /* set power save time to the scanner */
@@ -289,6 +299,9 @@ enum Avision_Option
   OPT_MESSAGE,           /* optional message from the scanner display */  
   OPT_NVRAM,             /* retrieve NVRAM values as pretty printed text */
   
+  OPT_PAPERLEN,          /* Use paper_length field to detect double feeds */
+  OPT_ADF_FLIP,          /* For flipping duplex, reflip the document */
+
   NUM_OPTIONS            /* must come last */
 };
 
@@ -356,7 +369,6 @@ typedef struct Avision_Device
   SANE_Bool inquiry_needs_software_colorpack;
   SANE_Bool inquiry_needs_line_pack;
   SANE_Bool inquiry_adf_need_mirror;
-  SANE_Bool inquiry_adf_need_mirror_rear;
   SANE_Bool inquiry_adf_bgr_order;
   SANE_Bool inquiry_light_detect;
   SANE_Bool inquiry_light_control;
@@ -402,10 +414,6 @@ typedef struct Avision_Device
   int scsi_buffer_size; /* nice to have SCSI buffer size */
   int read_stripe_size; /* stripes to be read at-a-time */
 
-  /* additional information - read delayed until sane_open() */
-  
-  SANE_Bool additional_probe;
-  
   /* film scanner atributes - maybe these should be in the scanner struct? */
   SANE_Range frame_range;
   SANE_Word current_frame;
@@ -491,13 +499,14 @@ typedef struct Avision_Scanner
 #define AVISION_SCSI_OP_TRANS_CALIB_GRAY    0x04
 #define AVISION_SCSI_OP_TRANS_CALIB_COLOR   0x05
 
-#define AVISION_FILTER_NONE	0
-#define AVISION_FILTER_RED	1
-#define AVISION_FILTER_GREEN	2
-#define AVISION_FILTER_BLUE	3
-#define AVISION_FILTER_RGB	4
-#define AVISION_FILTER_CMYK	5
-#define AVISION_FILTER_GRAY	6
+/* These apply to bitset1.  The values are 0 to 6, shifted 3 bits to the left */
+#define AVISION_FILTER_NONE	0x00
+#define AVISION_FILTER_RED	0x08
+#define AVISION_FILTER_GREEN	0x10
+#define AVISION_FILTER_BLUE	0x18
+#define AVISION_FILTER_RGB	0x20
+#define AVISION_FILTER_CMYK	0x28
+#define AVISION_FILTER_GRAY	0x30
 
 /* The SCSI structures that we have to send to an avision to get it to
    do various stuff... */
