@@ -3,7 +3,7 @@
 
    dc25.c 
 
-   $Id: dc25.c,v 1.17 2008-02-03 10:34:15 ellert-guest Exp $
+   $Id$
 
    This file (C) 1998 Peter Fales
 
@@ -89,7 +89,7 @@
 
  ***************************************************************************/
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -98,12 +98,12 @@
 #include <fcntl.h>
 #include <limits.h>
 
-#include "sane/sane.h"
-#include "sane/sanei.h"
-#include "sane/saneopts.h"
+#include "../include/sane/sane.h"
+#include "../include/sane/sanei.h"
+#include "../include/sane/saneopts.h"
 
 #define BACKEND_NAME	dc25
-#include "sane/sanei_backend.h"
+#include "../include/sane/sanei_backend.h"
 
 #include "dc25.h"
 
@@ -1845,12 +1845,12 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback UNUSEDARG authorize)
   DBG_INIT ();
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
   fp = sanei_config_open (DC25_CONFIG_FILE);
 
   DBG (127,
-       "sane_init() $Id: dc25.c,v 1.17 2008-02-03 10:34:15 ellert-guest Exp $\n");
+       "sane_init() $Id$\n");
 
   if (!fp)
     {
