@@ -9,6 +9,8 @@
    Copyright (C) 2010 Jack McGill <jmcgill85258@yahoo.com>
    Copyright (C) 2010 Andrey Loginov <avloginov@gmail.com>,
    		xerox travelscan device entry
+   Copyright (C) 2010 Chris Berry <s0457957@sms.ed.ac.uk> and Michael Rickmann <mrickma@gwdg.de>
+                 for Plustek Opticbook 3600 support
    
    This file is part of the SANE package.
    
@@ -124,7 +126,7 @@ static Genesys_Frontend Wolfson[] = {
   ,				/* 8: HP3670 */
   {DAC_WOLFSON_HP3670,  
    /* reg0  reg1  reg2  reg3 */
-     {0x00, 0x03, 0x05, 0x32}  /* reg3=0x32 for 100-300 dpi, 0x12 at 1200 */
+     {0x00, 0x03, 0x05, 0x32} /* reg3=0x32 for 100-300 dpi, 0x12 at 1200 */
    , {0x00, 0x00, 0x00} /* sign */
    , {0xba, 0xb8, 0xb8} /* offset */
    , {0x06, 0x05, 0x04} /* gain 4,3,2 at 1200 ?*/
@@ -137,7 +139,47 @@ static Genesys_Frontend Wolfson[] = {
    , {0xa0, 0xa0, 0xa0}
    , {0x07, 0x00, 0x00}
    }
-  ,		
+  ,
+  {DAC_CANONLIDE200,
+     {0x9d, 0x91, 0x00, 0x00}
+   , {0x00, 0x00, 0x00}
+   , {0x00, 0x3f, 0x00}  /* 0x00 0x3f 0x00 : offset/brigthness ? */
+   , {0x32, 0x04, 0x00}
+   , {0x00, 0x00, 0x00}
+   }
+  ,				/* KV-SS080 */
+  {DAC_KVSS080,
+     {0x00, 0x23, 0x24, 0x0f}
+   , {0x00, 0x00, 0x00}
+   , {0x80, 0x80, 0x80}	
+   , {0x4b, 0x4b, 0x4b}
+   , {0x00,0x00,0x00}
+   }
+  ,
+  {DAC_G4050,
+     {0x00, 0x23, 0x24, 0x1f}
+   , {0x00, 0x00, 0x00}
+   , {0x45, 0x45, 0x45}	/* 0x20, 0x21, 0x22 */
+   , {0x4b, 0x4b, 0x4b} /* 0x28, 0x29, 0x2a */
+   , {0x00,0x00,0x00}
+   }
+  ,
+  {DAC_CANONLIDE110,
+     {0x80, 0x8a, 0x23, 0x4c}
+   , {0x00, 0xca, 0x94}
+   , {0x00, 0x00, 0x00}
+   , {0x00, 0x00, 0x00}
+   , {0x00, 0x00, 0x00}
+   }
+  ,
+  {DAC_PLUSTEK_3600,
+     {0x70, 0x80, 0x00, 0x00}
+   , {0x00, 0x00, 0x00}
+   , {0x00, 0x00, 0x00}
+   , {0x3f, 0x3d, 0x3d}
+   , {0x00, 0x00, 0x00}
+   }
+  ,
 };
 
 
@@ -393,7 +435,136 @@ static Genesys_Sensor Sensor[] = {
    1.0, 1.0, 1.0,
    NULL, NULL, NULL}
   ,
-
+  /* CANONLIDE200 */
+  {CIS_CANONLIDE200,
+   1200,	/* optical resolution */
+   87,		/* black pixels */
+   16,		/* dummy pixels */
+   303,		
+   10272,		
+   210,
+   200,
+   {0x00, 0x00, 0x00, 0x00},
+   /* reg 0x10 - 0x1d */
+   {0x01, 0xc0, 0x01, 0xc0, 0x01, 0x40, /* EXPR/EXPG/EXPB */
+    0x10, 0x08, 0x00, 0xff, 0x34, 0x00, 0x02, 0x04 },
+   /* reg 0x52 - 0x5e */
+   {0x03, 0x07,
+    0x00, 0x00, 0x00, 0x00,
+    0x2a, 0xe1,
+    0x55,		
+    0x00, 0x00, 0x00, 
+    0x41	
+    }
+   ,
+   1.7, 1.7, 1.7,
+   NULL, NULL, NULL}
+  ,
+  /* CANONLIDE100 */
+  {CIS_CANONLIDE100,
+   1200,	/* optical resolution */
+   87,		/* black pixels */
+   16,		/* dummy pixels 16 */
+   303,		/* 303 */
+   10272,       /* 10272 */
+   210,
+   200,
+   {0x00, 0x00, 0x00, 0x00},
+   /* reg 0x10 - 0x15 */
+   {0x01, 0xc1, 0x01, 0x26, 0x00, 0xe5, /* EXPR/EXPG/EXPB */
+   /* reg 0x16 - 0x1d 0x19=0x50*/
+    0x10, 0x08, 0x00, 0x50, 0x34, 0x00, 0x02, 0x04 },
+   /* reg 0x52 - 0x5e */
+   {0x03, 0x07,
+    0x00, 0x00, 0x00, 0x00,
+    0x2a, 0xe1,
+    0x55,		
+    0x00, 0x00, 0x00, 
+    0x41	
+    }
+   ,
+   1.7, 1.7, 1.7,
+   NULL, NULL, NULL}
+  ,
+  {CCD_KVSS080,
+   600,
+   38, /* black pixels on left */
+   38, /* 36 dummy pixels */
+   152,
+   5376, /* 5100-> 5200 */
+   160, /* TAU white ref */
+   160, /* gain white ref */
+   /* 08    09    0a    0b */
+   {0x00, 0x00, 0x00, 0x6a} ,
+   /* 10    11    12    13    14    15    16    17    18    19    1a    1b    1c    1d */
+   {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x33, 0x1c, 0x00, 0x2a, 0x2c, 0x00, 0x20, 0x04} , /* 18=00 at 600 dpi */
+   /* 52    53    54    55    56    57    58    59   5a    5b     5c    5d    5e */
+   {0x0c, 0x0f, 0x00, 0x03, 0x06, 0x09, 0x6b, 0x00, 0xc0, 0x00, 0x00, 0x00, 0x23} ,
+   1.0, 1.0, 1.0,
+   NULL, NULL, NULL}
+  ,
+  {CCD_G4050,
+   4800,
+   48,
+   28,
+   152,
+   20400, /* 20400 only half part of the sensor is used */
+   210,
+   230,
+   /* 08    09    0a    0b */
+   {0x00, 0x00, 0x18, 0x69} ,
+   /* 10    11    12    13    14    15    16    17    18    19    1a    1b    1c    1d */
+   {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x33, 0x0c, 0x01, 0x2a, 0x30, 0x00, 0x00, 0x08} ,
+   /* 52    53    54    55    56    57    58    59   5a    5b     5c    5d    5e */
+   {0x0e, 0x11, 0x02, 0x05, 0x08, 0x0b, 0x6b, 0x00, 0xc0, 0x00, 0x00, 0x00, 0x6f} ,
+   1.0, 1.0, 1.0,
+   NULL, NULL, NULL}
+  ,
+  /* CANONLIDE110 */
+  {CIS_CANONLIDE110,
+   2400,	/* optical resolution */
+   87,		/* black pixels */
+   16,		/* dummy pixels 16 */
+   303,		/* 303 */
+   5168*4,
+   210,
+   200,
+   {0x00, 0x00, 0x00, 0x00},
+   /* reg 0x10 - 0x15 : EXPR, EXPG and EXPB */
+   {0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+   /* reg 0x16 - 0x1d */
+    0x10, 0x04, 0x00, 0x01, 0x30, 0x00, 0x02, 0x01 },
+   /* reg 0x52 - 0x5e */
+   {
+    0x00, 0x02, 0x04, 0x06, 0x04, 0x04, 0x04, 0x04,
+    0x1a, 0x00, 0xc0, 0x00, 0x00
+    }
+   ,
+   1.7, 1.7, 1.7,
+   NULL, NULL, NULL}
+  ,
+  {CCD_PLUSTEK_3600, 1200,
+   /*TODO: find a good reason for keeping all three following variables*/
+   87,				/*(black) */
+   87,				/* (dummy) */
+   0,				/* (startxoffset) */
+   10100,			/*sensor_pixels */
+   210,
+   230,
+   {0x00, 0x00, 0x00, 0x00},
+   {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x33, 0x0b, 0x11, 0x2a,
+    0x00, 0x00, 0x00, 0xc4	/* TODO(these do no harm, but may be neccessery for CCD) */
+   },
+   {0x07, 0x0a,
+    0x0c, 0x00, 0x02, 0x06,	/*[GB](HI|LOW) not needed for cis */
+    0x22, 0x69,
+    0x40,			/*TODO: bit7 */
+    0x00, 0x00, 0x00, 0x02	/*TODO (these do no harm, but may be neccessery for CCD) */
+   }
+   ,
+   1.0, 1.0, 1.0,
+   NULL, NULL, NULL}
+  ,
 };
 
 /** for General Purpose Output specific settings:
@@ -492,6 +663,33 @@ static Genesys_Gpo Gpo[] = {
    {0xfb, 0x00}, /* 6e, 6f */
   }
   ,
+  /* CANONLIDE200 */
+  {GPO_CANONLIDE200,
+   {0xfb, 0x20},	/* 0xfb when idle , 0xf9/0xe9 (1200) when scanning */
+   {0xff, 0x00},
+  }
+  ,
+  {GPO_KVSS080,
+   {0xf5, 0x20},
+   {0x7e, 0xa1},
+  }
+  ,
+  {GPO_G4050,
+   {0x20, 0x00},
+   {0xfc, 0x00},
+  }
+  ,
+  /* CANONLIDE110 */
+  {GPO_CANONLIDE110,
+   {0xfb, 0x20},
+   {0xff, 0x00},
+  }
+  ,
+  /* Plustek 3600 */
+  {GPO_PLUSTEK_3600,
+   {0x02, 0x00},
+   {0x1e, 0x80},
+  }
 };
 
 static Genesys_Motor Motor[] = {
@@ -697,7 +895,7 @@ static Genesys_Motor Motor[] = {
   {MOTOR_DSMOBILE_600,			/* Pentax DSmobile 600 */
    750,
    1500,
-   1,
+   2,
    1,
    {{{
      6666,
@@ -706,12 +904,86 @@ static Genesys_Motor Motor[] = {
      0.8,
      },
     {
-     11000,
-     11000,
-     2,
+     6666,
+     3700,
+     8,
      0.8,
      },},},
   },
+  {MOTOR_CANONLIDE100,		/* Canon LiDE 100 */
+   1200,
+   4800,
+   2,   /* maximum step type count */
+   1,   /* maximum power modes count */
+   { /* motor slopes */
+	   { /* power mode 0 */
+		   {   3000,   1000, 127, 0.50}, /* full step */
+    		   {   3000,   1500, 127, 0.50}, /* half step */
+    		   { 3*2712, 3*2712, 16, 0.80}, /* quarter step 0.75*2712 */
+	   },
+    },
+  },
+  {MOTOR_CANONLIDE200,		/* Canon LiDE 200 */
+   1200,
+   4800,
+   2,
+   1,
+   { /* motor slopes */
+	   { /* power mode 0 */
+		   {   3000,   1000, 127, 0.50}, /* full step */
+    		   {   3000,   1500, 127, 0.50}, /* half step */
+    		   { 3*2712, 3*2712, 16, 0.80}, /* quarter step 0.75*2712 */
+	   },
+    },
+  },
+  {MOTOR_KVSS080,
+   1200,
+   1200,
+   2,
+   1,
+   { /* motor slopes */
+	   { /* power mode 0 */
+     		{ 22222, 500, 246, 0.5 }, /* max speed / dpi * base dpi => exposure */
+     		{ 22222, 500, 246, 0.5 },
+     		{ 22222, 500, 246, 0.5 },
+    	   },
+   },
+  },
+  {MOTOR_G4050,
+   2400,
+   4800,
+   2,
+   1,
+   { /* motor slopes */
+	   { /* power mode 0 */
+     		{ 3961, 240, 246, 0.8 }, /* full step   */
+     		{ 3961, 240, 246, 0.8 }, /* half step   */
+     		{ 3961, 240, 246, 0.8 }, /* quarter step */
+    	   },
+   },
+  },
+  {MOTOR_CANONLIDE110,		/* Canon LiDE 110 */
+   4800,
+   9600,
+   1,   /* maximum step type count */
+   1,   /* maximum power modes count */
+   { /* motor slopes */
+	   { /* power mode 0 */
+		   {   3000,   1000, 256, 0.50}, /* full step */
+	   },
+    },
+  },
+  {MOTOR_PLUSTEK_3600,		/* PLUSTEK 3600 */
+   1200,
+   2400,
+   1,
+   1,
+   {
+     {
+       { 3500, 1300, 60, 0.8 },
+       { 3500, 3250, 60, 0.8 },
+     },
+   },},
 };
 
 /* here we have the various device settings...
@@ -821,6 +1093,483 @@ static Genesys_Model canon_lide_50_model = {
   400
 };
 
+static Genesys_Model panasonic_kvss080_model = {
+  "panasonic-kv-ss080",		/* Name */
+  "Panasonic",			/* Device vendor string */
+  "KV-SS080",			/* Device model name */
+  GENESYS_GL843,
+  NULL,
+
+  { 600, /* 500, 400,*/ 300, 200, 150, 100, 75, 0},	/* possible x-resolutions */
+  { 1200, 600, /* 500, 400, */ 300, 200, 150, 100, 75, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (7.2),		/* Start of scan area in mm  (x) */
+  SANE_FIX (14.7),		/* Start of scan area in mm (y) */
+  SANE_FIX (217.7),		/* Size of scan area in mm (x) */
+  SANE_FIX (300.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (9.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (0.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (0.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Size of scan area after paper sensor stops
+				   sensing document in mm */
+  SANE_FIX (0.0),		/* Amount of feeding needed to eject document 
+				   after finishing scanning in mm */
+
+  0, 8, 16,			/* RGB CCD Line-distance correction in pixel */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+
+  SANE_FALSE,			/* Is this a CIS scanner? */
+  SANE_FALSE,			/* Is this a sheetfed scanner? */
+  CCD_KVSS080,
+  DAC_KVSS080,
+  GPO_KVSS080,
+  MOTOR_KVSS080,
+  GENESYS_FLAG_LAZY_INIT |
+  GENESYS_FLAG_SKIP_WARMUP |
+  GENESYS_FLAG_OFFSET_CALIBRATION |
+  GENESYS_FLAG_CUSTOM_GAMMA,
+  GENESYS_HAS_SCAN_SW ,
+  100,
+  100
+};
+
+static Genesys_Model hpg4010_model = {
+  "hewlett-packard-scanjet-g4010",	/* Name */
+  "Hewlett Packard",			/* Device vendor string */
+  "ScanJet G4010",			/* Device model name */
+  GENESYS_GL843,
+  NULL,
+
+  { 2400, 1200, 600, 400, 300, 200, 150, 100, 0}, /* TODO when settled down, add 800 and 1600 */
+  { 2400, 1200, 600, 400, 300, 200, 150, 100, 0}, /* TODO 4800 available */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (6.55),		/* Start of scan area in mm  (x) */
+  SANE_FIX (15.00),		/* Start of scan area in mm (y) */
+  SANE_FIX (223.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (315.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (3.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (0.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Size of scan area after paper sensor stops
+				   sensing document in mm */
+  SANE_FIX (0.0),		/* Amount of feeding needed to eject document 
+				   after finishing scanning in mm */
+
+  0, 32, 48,			/* RGB CCD Line-distance correction in pixel */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+
+  SANE_FALSE,			/* Is this a CIS scanner? */
+  SANE_FALSE,			/* Is this a sheetfed scanner? */
+  CCD_G4050,
+  DAC_G4050,
+  GPO_G4050,
+  MOTOR_G4050,
+  GENESYS_FLAG_NO_CALIBRATION |
+  GENESYS_FLAG_LAZY_INIT | 	/* Which flags are needed for this scanner? */
+  GENESYS_FLAG_SKIP_WARMUP | 
+  GENESYS_FLAG_OFFSET_CALIBRATION |
+  GENESYS_FLAG_CUSTOM_GAMMA,
+  GENESYS_HAS_SCAN_SW | GENESYS_HAS_FILE_SW | GENESYS_HAS_COPY_SW,
+  280,
+  400
+};
+
+static Genesys_Model hpg4050_model = {
+  "hewlett-packard-scanjet-g4050",	/* Name */
+  "Hewlett Packard",			/* Device vendor string */
+  "ScanJet G4050",			/* Device model name */
+  GENESYS_GL843,
+  NULL,
+
+  { 2400, 1200, 600, 400, 300, 200, 150, 100, 0}, /* TODO when settled down, add 800 and 1600 */
+  { 2400, 1200, 600, 400, 300, 200, 150, 100, 0}, /* TODO 4800 available */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (6.55),		/* Start of scan area in mm  (x) */
+  SANE_FIX (15.00),		/* Start of scan area in mm (y) */
+  SANE_FIX (223.0),		/* Size of scan area in mm (x) */
+  SANE_FIX (315.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (3.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (0.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Size of scan area after paper sensor stops
+				   sensing document in mm */
+  SANE_FIX (0.0),		/* Amount of feeding needed to eject document 
+				   after finishing scanning in mm */
+
+  0, 32, 48,			/* RGB CCD Line-distance correction in pixel */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+
+  SANE_FALSE,			/* Is this a CIS scanner? */
+  SANE_FALSE,			/* Is this a sheetfed scanner? */
+  CCD_G4050,
+  DAC_G4050,
+  GPO_G4050,
+  MOTOR_G4050,
+  GENESYS_FLAG_NO_CALIBRATION |
+  GENESYS_FLAG_LAZY_INIT | 	/* Which flags are needed for this scanner? */
+  GENESYS_FLAG_SKIP_WARMUP | 
+  GENESYS_FLAG_OFFSET_CALIBRATION |
+  GENESYS_FLAG_CUSTOM_GAMMA,
+  GENESYS_HAS_SCAN_SW | GENESYS_HAS_FILE_SW | GENESYS_HAS_COPY_SW,
+  280,
+  400
+};
+
+
+static Genesys_Model canon_lide_100_model = {
+  "canon-lide-100",		/* Name */
+  "Canon",			/* Device vendor string */
+  "LiDE 100",			/* Device model name */
+  GENESYS_GL847,
+  NULL,
+
+  {1200, 600, 400, 300, 200, 150, 100, 75, 0},	/* possible x-resolutions */
+  {1200, 600, 400, 300, 200, 150, 100, 75, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (1.1),		/* Start of scan area in mm (x) */
+  SANE_FIX (8.3),		/* Start of scan area in mm (y) */
+  SANE_FIX (216.07),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (3.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (0.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Size of scan area after paper sensor stops
+				   sensing document in mm */
+  SANE_FIX (0.0),		/* Amount of feeding needed to eject document 
+				   after finishing scanning in mm */
+
+  0, 0, 0,			/* RGB CCD Line-distance correction in pixel */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+
+  SANE_TRUE,			/* Is this a CIS scanner? */
+  SANE_FALSE,			/* Is this a sheetfed scanner? */
+  CIS_CANONLIDE100,
+  DAC_CANONLIDE200,
+  GPO_CANONLIDE200,
+  MOTOR_CANONLIDE100,	
+  /* Which flags are needed for this scanner? */
+      GENESYS_FLAG_SKIP_WARMUP
+    | GENESYS_FLAG_ODD_EVEN_CIS
+    | GENESYS_FLAG_MUST_WAIT
+    | GENESYS_FLAG_OFFSET_CALIBRATION
+    | GENESYS_FLAG_DARK_CALIBRATION
+    | GENESYS_FLAG_CUSTOM_GAMMA,
+  GENESYS_HAS_SCAN_SW | GENESYS_HAS_COPY_SW | GENESYS_HAS_EMAIL_SW | GENESYS_HAS_FILE_SW,
+  60,
+  400
+};
+
+static Genesys_Model canon_lide_110_model = {
+  "canon-lide-110",		/* Name */
+  "Canon",			/* Device vendor string */
+  "LiDE 110",			/* Device model name */
+  GENESYS_GL124,
+  NULL,
+
+  {2400, 1200, 600, /* 400,*/ 300, 150, 100, 75, 0},	/* possible x-resolutions */
+  {2400, 1200, 600, /* 400,*/ 300, 150, 100, 75, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (3.6),		/* Start of scan area in mm (x) */
+  SANE_FIX (8.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (213.80),		/* Size of scan area in mm (x) */
+  SANE_FIX (300.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (1.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (0.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Size of scan area after paper sensor stops
+				   sensing document in mm */
+  SANE_FIX (0.0),		/* Amount of feeding needed to eject document 
+				   after finishing scanning in mm */
+
+  0, 0, 0,			/* RGB CCD Line-distance correction in pixel */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+
+  SANE_TRUE,			/* Is this a CIS scanner? */
+  SANE_FALSE,			/* Is this a sheetfed scanner? */
+  CIS_CANONLIDE110,
+  DAC_CANONLIDE110,
+  GPO_CANONLIDE110,
+  MOTOR_CANONLIDE110,	
+      GENESYS_FLAG_SKIP_WARMUP
+    | GENESYS_FLAG_OFFSET_CALIBRATION
+    | GENESYS_FLAG_DARK_CALIBRATION
+    | GENESYS_FLAG_CUSTOM_GAMMA,
+  GENESYS_HAS_SCAN_SW | GENESYS_HAS_COPY_SW | GENESYS_HAS_EMAIL_SW | GENESYS_HAS_FILE_SW,
+  50,
+  400
+};
+
+
+static Genesys_Model canon_lide_210_model = {
+  "canon-lide-210",		/* Name */
+  "Canon",			/* Device vendor string */
+  "LiDE 210",			/* Device model name */
+  GENESYS_GL124,
+  NULL,
+
+  {/*2400, */ 1200, 600, /* 400,*/ 300, 150, 100, 75, 0},	/* possible x-resolutions */
+  {/*2400, */ 1200, 600, /* 400,*/ 300, 150, 100, 75, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (3.6),		/* Start of scan area in mm (x) */
+  SANE_FIX (8.5),		/* Start of scan area in mm (y) */
+  SANE_FIX (213.80),		/* Size of scan area in mm (x) */
+  SANE_FIX (300.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (0.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Size of scan area after paper sensor stops
+				   sensing document in mm */
+  SANE_FIX (0.0),		/* Amount of feeding needed to eject document 
+				   after finishing scanning in mm */
+
+  0, 0, 0,			/* RGB CCD Line-distance correction in pixel */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+
+  SANE_TRUE,			/* Is this a CIS scanner? */
+  SANE_FALSE,			/* Is this a sheetfed scanner? */
+  CIS_CANONLIDE110,
+  DAC_CANONLIDE110,
+  GPO_CANONLIDE110,
+  MOTOR_CANONLIDE110,	
+      GENESYS_FLAG_SKIP_WARMUP
+    | GENESYS_FLAG_OFFSET_CALIBRATION
+    | GENESYS_FLAG_DARK_CALIBRATION
+    | GENESYS_FLAG_CUSTOM_GAMMA,
+  GENESYS_HAS_SCAN_SW | GENESYS_HAS_COPY_SW | GENESYS_HAS_EMAIL_SW | GENESYS_HAS_FILE_SW,
+  50,
+  400
+};
+
+static Genesys_Model canon_5600f_model = {
+  "canon-5600f",		/* Name */
+  "Canon",			/* Device vendor string */
+  "5600F",			/* Device model name */
+  GENESYS_GL847,
+  NULL,
+
+  {1200, 600, 400, 300, 200, 150, 100, 75, 0},	/* possible x-resolutions */
+  {1200, 600, 400, 300, 200, 150, 100, 75, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (1.1),		/* Start of scan area in mm  (x) */
+  SANE_FIX (8.3),		/* Start of scan area in mm (y) */
+  SANE_FIX (216.07),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (3.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (0.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Size of scan area after paper sensor stops
+				   sensing document in mm */
+  SANE_FIX (0.0),		/* Amount of feeding needed to eject document 
+				   after finishing scanning in mm */
+
+  0, 0, 0,			/* RGB CCD Line-distance correction in pixel */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+
+  SANE_TRUE,			/* Is this a CIS scanner? */
+  SANE_FALSE,			/* Is this a sheetfed scanner? */
+  CIS_CANONLIDE200,
+  DAC_CANONLIDE200,
+  GPO_CANONLIDE200,
+  MOTOR_CANONLIDE200,
+  GENESYS_FLAG_UNTESTED		/* not working yet */
+    | GENESYS_FLAG_MUST_WAIT
+    | GENESYS_FLAG_SKIP_WARMUP
+    | GENESYS_FLAG_ODD_EVEN_CIS
+    | GENESYS_FLAG_OFFSET_CALIBRATION
+    | GENESYS_FLAG_DARK_CALIBRATION
+    | GENESYS_FLAG_CUSTOM_GAMMA,
+  GENESYS_HAS_SCAN_SW | GENESYS_HAS_COPY_SW | GENESYS_HAS_EMAIL_SW | GENESYS_HAS_FILE_SW,
+  60,
+  400
+};
+
+static Genesys_Model canon_lide_700f_model = {
+  "canon-lide-700f",		/* Name */
+  "Canon",			/* Device vendor string */
+  "LiDE 700F",			/* Device model name */
+  GENESYS_GL847,
+  NULL,
+
+  {1200, 600, 400, 300, 200, 150, 100, 75, 0},	/* possible x-resolutions */
+  {1200, 600, 400, 300, 200, 150, 100, 75, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (1.1),		/* Start of scan area in mm  (x) */
+  SANE_FIX (8.3),		/* Start of scan area in mm (y) */
+  SANE_FIX (216.07),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (3.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (0.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Size of scan area after paper sensor stops
+				   sensing document in mm */
+  SANE_FIX (0.0),		/* Amount of feeding needed to eject document 
+				   after finishing scanning in mm */
+
+  0, 0, 0,			/* RGB CCD Line-distance correction in pixel */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+
+  SANE_TRUE,			/* Is this a CIS scanner? */
+  SANE_FALSE,			/* Is this a sheetfed scanner? */
+  CIS_CANONLIDE200,
+  DAC_CANONLIDE200,
+  GPO_CANONLIDE200,
+  MOTOR_CANONLIDE200,
+  GENESYS_FLAG_UNTESTED		/* not working yet */
+    | GENESYS_FLAG_MUST_WAIT
+    | GENESYS_FLAG_SKIP_WARMUP
+    | GENESYS_FLAG_ODD_EVEN_CIS
+    | GENESYS_FLAG_OFFSET_CALIBRATION
+    | GENESYS_FLAG_DARK_CALIBRATION
+    | GENESYS_FLAG_CUSTOM_GAMMA,
+  GENESYS_HAS_SCAN_SW | GENESYS_HAS_COPY_SW | GENESYS_HAS_EMAIL_SW | GENESYS_HAS_FILE_SW,
+  60,
+  400
+};
+
+
+
+static Genesys_Model canon_lide_200_model = {
+  "canon-lide-200",		/* Name */
+  "Canon",			/* Device vendor string */
+  "LiDE 200",			/* Device model name */
+  GENESYS_GL847,
+  NULL,
+
+  {1200, 600, 400, 300, 200, 150, 100, 75, 0},	/* possible x-resolutions */
+  {1200, 600, 400, 300, 200, 150, 100, 75, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (1.1),		/* Start of scan area in mm  (x) */
+  SANE_FIX (8.3),		/* Start of scan area in mm (y) */
+  SANE_FIX (216.07),		/* Size of scan area in mm (x) */
+  SANE_FIX (299.0),		/* Size of scan area in mm (y) */
+
+  SANE_FIX (3.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (0.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (100.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Size of scan area after paper sensor stops
+				   sensing document in mm */
+  SANE_FIX (0.0),		/* Amount of feeding needed to eject document 
+				   after finishing scanning in mm */
+
+  0, 0, 0,			/* RGB CCD Line-distance correction in pixel */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+
+  SANE_TRUE,			/* Is this a CIS scanner? */
+  SANE_FALSE,			/* Is this a sheetfed scanner? */
+  CIS_CANONLIDE200,
+  DAC_CANONLIDE200,
+  GPO_CANONLIDE200,
+  MOTOR_CANONLIDE200,
+      GENESYS_FLAG_SKIP_WARMUP
+    | GENESYS_FLAG_MUST_WAIT
+    | GENESYS_FLAG_ODD_EVEN_CIS
+    | GENESYS_FLAG_OFFSET_CALIBRATION
+    | GENESYS_FLAG_DARK_CALIBRATION
+    | GENESYS_FLAG_CUSTOM_GAMMA,
+  GENESYS_HAS_SCAN_SW | GENESYS_HAS_COPY_SW | GENESYS_HAS_EMAIL_SW | GENESYS_HAS_FILE_SW,
+  60,
+  400
+};
+
+
 static Genesys_Model canon_lide_60_model = {
   "canon-lide-60",		/* Name */
   "Canon",			/* Device vendor string */
@@ -918,6 +1667,7 @@ static Genesys_Model hp2300c_model = {
   GPO_HP2300,
   MOTOR_HP2300,
   GENESYS_FLAG_14BIT_GAMMA
+    | GENESYS_FLAG_SKIP_WARMUP
     | GENESYS_FLAG_LAZY_INIT
     | GENESYS_FLAG_SEARCH_START
     | GENESYS_FLAG_DARK_CALIBRATION
@@ -937,8 +1687,8 @@ Genesys_Model hp2400c_model = {
   GENESYS_GL646,
   NULL,
 
-  {1200, 600, 300, 150, 100, 75, 50, 0},	/* possible x-resolutions */
-  {1200, 600, 300, 150, 100, 75, 50, 0},	/* possible y-resolutions */
+  {1200, 600, 300, 150, 100, 50, 0},	/* possible x-resolutions */
+  {1200, 600, 300, 150, 100, 50, 0},	/* possible y-resolutions */
   {16, 8, 0},			/* possible depths in gray mode */
   {16, 8, 0},			/* possible depths in color mode */
 
@@ -964,7 +1714,7 @@ Genesys_Model hp2400c_model = {
 
   0, 24, 48,			/* RGB CCD Line-distance correction in pixel */
 
-  COLOR_ORDER_BGR,		/* Order of the CCD/CIS colors */
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
 
   SANE_FALSE,			/* Is this a CIS scanner? */
   SANE_FALSE,			/* Is this a sheetfed scanner? */
@@ -1881,30 +2631,100 @@ static Genesys_Model xerox_travelscanner_model = {
   400
 };
 
+static Genesys_Model plustek_3600_model = {
+  "plustek-opticbook-3600",	/* Name */
+  "PLUSTEK",			/* Device vendor string */
+  "OpticBook 3600",		/* Device model name */
+  GENESYS_GL841,
+  NULL,
+  {/*1200,*/ 600, 400, 300, 200, 150, 100, 75, 0},		/* possible x-resolutions */
+  {/*2400,*/ 1200, 600, 400, 300, 200, 150, 100, 75, 0},	/* possible y-resolutions */
+  {16, 8, 0},			/* possible depths in gray mode */
+  {16, 8, 0},			/* possible depths in color mode */
+
+  SANE_FIX (0.42),/*SANE_FIX (0.42),		 Start of scan area in mm  (x) */
+  SANE_FIX (6.75),/*SANE_FIX (7.9),		 Start of scan area in mm (y) */
+  SANE_FIX (216.0),/*SANE_FIX (216.0),		 Size of scan area in mm (x) */
+  SANE_FIX (297.0),/*SANE_FIX (297.0),		 Size of scan area in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in mm (y) */
+  SANE_FIX (0.0),		/* Start of black mark in mm (x) */
+
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Start of scan area in TA mode in mm (y) */
+  SANE_FIX (0.0),		/* Size of scan area in TA mode in mm (x) */
+  SANE_FIX (0.0),		/* Size of scan area in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Start of white strip in TA mode in mm (y) */
+
+  SANE_FIX (0.0),		/* Size of scan area after paper sensor stops
+				   sensing document in mm */
+  SANE_FIX (0.0),		/* Amount of feeding needed to eject document
+				   after finishing scanning in mm */
+ 
+  0, 24, 48,			/* RGB CCD Line-distance correction in pixel */
+
+  COLOR_ORDER_RGB,		/* Order of the CCD/CIS colors */
+
+  SANE_FALSE,			/* Is this a CIS scanner? */
+  SANE_FALSE,			/* Is this a sheetfed scanner? */
+  CCD_PLUSTEK_3600,
+  DAC_PLUSTEK_3600,
+  GPO_PLUSTEK_3600,
+  MOTOR_PLUSTEK_3600,
+  GENESYS_FLAG_UNTESTED		/* not fully working yet */
+	  | GENESYS_FLAG_CUSTOM_GAMMA
+	  | GENESYS_FLAG_SKIP_WARMUP
+	  | GENESYS_FLAG_DARK_CALIBRATION
+	  | GENESYS_FLAG_OFFSET_CALIBRATION
+  	  | GENESYS_FLAG_LAZY_INIT
+  	  | GENESYS_FLAG_HALF_CCD_MODE,/*
+      | GENESYS_FLAG_NO_CALIBRATION,*/
+  GENESYS_HAS_NO_BUTTONS,
+  7,
+  200
+};
+ 
 
 
 static Genesys_USB_Device_Entry genesys_usb_device_list[] = {
+  /* GL646 devices */
   {0x03f0, 0x0901, &hp2300c_model},
   {0x03f0, 0x0a01, &hp2400c_model},
   {0x03f0, 0x1405, &hp3670c_model},
   {0x0461, 0x0377, &medion_md5345_model},
   {0x04a7, 0x0229, &visioneer_7100_model},
   {0x04a7, 0x0426, &visioneer_xp200_model},
+  {0x0638, 0x0a10, &umax_astra_4500_model},
+  {0x07b3, 0x0600, &plustek_st12_model},
+  {0x07b3, 0x0601, &plustek_st24_model},
+  /* GL841 devices */
   {0x04a7, 0x0474, &visioneer_xp300_model},
   {0x04a7, 0x0494, &visioneer_roadwarrior_model},
   {0x04a7, 0x049b, &visioneer_xp100_r3_model},
   {0x04a7, 0x04ac, &xerox_travelscanner_model},
   {0x04a9, 0x2213, &canon_lide_50_model},
   {0x04a9, 0x221c, &canon_lide_60_model},
-  {0x0638, 0x0a10, &umax_astra_4500_model},
-  {0x07b3, 0x0600, &plustek_st12_model},
-  {0x07b3, 0x0601, &plustek_st24_model},
+  {0x07b3, 0x0900, &plustek_3600_model},
   {0x0a17, 0x3210, &pentax_dsmobile_600_model},
+  {0x04f9, 0x2038, &pentax_dsmobile_600_model}, /* clone, only usb id is different */
   {0x0a82, 0x4800, &syscan_docketport_485_model},
   {0x0a82, 0x4802, &syscan_docketport_465_model},
   {0x0a82, 0x4803, &syscan_docketport_665_model},
   {0x0a82, 0x480c, &syscan_docketport_685_model},
   {0x1dcc, 0x4810, &dct_docketport_487_model},
   {0x1dcc, 0x4812, &syscan_docketport_467_model},
+  /* GL847 devices */
+  {0x04a9, 0x1904, &canon_lide_100_model},
+  {0x04a9, 0x1905, &canon_lide_200_model},
+  {0x04a9, 0x1906, &canon_5600f_model},
+  {0x04a9, 0x1907, &canon_lide_700f_model},
+  /* GL843 devices */
+  {0x04da, 0x100f, &panasonic_kvss080_model},
+  {0x03f0, 0x4505, &hpg4010_model},
+  {0x03f0, 0x4605, &hpg4050_model},
+  /* GL124 devices */
+  {0x04a9, 0x1909, &canon_lide_110_model},
+  {0x04a9, 0x190a, &canon_lide_210_model},
   {0, 0, NULL}
 };
