@@ -107,6 +107,7 @@
 #define REG05_DPIHW_600	0x00
 #define REG05_DPIHW_1200	0x40
 #define REG05_DPIHW_2400	0x80
+#define REG05_DPIHW_4800	0xc0
 #define REG05_GMMTYPE	0x30
 #define REG05_GMM14BIT  0x10
 #define REG05_GMMENB	0x08
@@ -471,12 +472,16 @@ static Sensor_Master sensor_master[] = {
   /* HP 2400 master settings */
   {CCD_HP2400,  50, SANE_TRUE ,   50,   675,  200, 4, 42, NULL, SANE_FALSE, 0x3f, 0x02},
   {CCD_HP2400, 100, SANE_TRUE ,  100,  7211,  400, 4, 42, NULL, SANE_FALSE, 0x3f, 0x02},
+  {CCD_HP2400, 150, SANE_TRUE ,  150,  7211,  600, 4, 42, NULL, SANE_FALSE, 0x3f, 0x02},
   {CCD_HP2400, 300, SANE_TRUE ,  300,  8751, 1200, 4, 42, NULL, SANE_FALSE, 0x3f, 0x02},
   {CCD_HP2400, 600, SANE_TRUE , 1200, 18760, 1200, 2, 42, NULL, SANE_FALSE, 0x3f, 0x02},
+  {CCD_HP2400,1200, SANE_TRUE , 1200, 18760, 1200, 1, 42, NULL, SANE_FALSE, 0x3f, 0x02},
   {CCD_HP2400,  50, SANE_FALSE,   50,   675,  200, 4, 42, NULL, SANE_FALSE, 0x3f, 0x02},
   {CCD_HP2400, 100, SANE_FALSE,  100,  7211,  400, 4, 42, NULL, SANE_FALSE, 0x3f, 0x02},
+  {CCD_HP2400, 150, SANE_FALSE,  150,  7211,  600, 4, 42, NULL, SANE_FALSE, 0x3f, 0x02},
   {CCD_HP2400, 300, SANE_FALSE,  300,  8751, 1200, 4, 42, NULL, SANE_FALSE, 0x3f, 0x02},
   {CCD_HP2400, 600, SANE_FALSE, 1200, 18760, 1200, 2, 42, NULL, SANE_FALSE, 0x3f, 0x02},
+  {CCD_HP2400,1200, SANE_FALSE, 1200, 18760, 1200, 1, 42, NULL, SANE_FALSE, 0x3f, 0x02},
 
   /* XP 200 master settings */
   {CIS_XP200 ,  75, SANE_TRUE ,  75,  5700,  75, 1, 42, xp200_color, SANE_FALSE, 0x00, 0x11},
@@ -554,13 +559,17 @@ static Motor_Master motor_master[] = {
 
   /* HP2400/G2410 motor settings base motor dpi = 600 */
   {MOTOR_HP2400,  50, SANE_TRUE ,  50, HALF_STEP, SANE_FALSE, SANE_FALSE, 63,  4,  1382,   800, 192, 4905,  337, 0.30, 0.4, 192},
-  {MOTOR_HP2400, 100, SANE_TRUE , 100, FULL_STEP, SANE_FALSE, SANE_TRUE,  63, 120, 8736,   601, 192, 4905,  337, 0.30, 0.4, 192},
+  {MOTOR_HP2400, 100, SANE_TRUE , 100, HALF_STEP, SANE_FALSE, SANE_TRUE,  63, 120, 8736,   601, 192, 4905,  337, 0.30, 0.4, 192},
+  {MOTOR_HP2400, 150, SANE_TRUE , 150, HALF_STEP, SANE_FALSE, SANE_TRUE,  63, 120, 8736,   601, 192, 4905,  337, 0.30, 0.4, 192},
   {MOTOR_HP2400, 300, SANE_TRUE , 300, HALF_STEP, SANE_FALSE, SANE_TRUE , 63, 32, 16703,  2188, 192, 4905,  337, 0.30, 0.4, 192},
   {MOTOR_HP2400, 600, SANE_TRUE , 600, FULL_STEP, SANE_FALSE, SANE_TRUE , 63,  3, 18761, 18761, 192, 4905,  627, 0.30, 0.4, 192},
+  {MOTOR_HP2400,1200, SANE_TRUE ,1200, HALF_STEP, SANE_FALSE, SANE_TRUE , 63,  3, 18761, 18761, 192, 4905,  627, 0.30, 0.4, 192},
   {MOTOR_HP2400,  50, SANE_FALSE,  50, HALF_STEP, SANE_FALSE, SANE_FALSE, 63,  4,  1382,   800, 192, 4905,  337, 0.30, 0.4, 192},
-  {MOTOR_HP2400, 100, SANE_FALSE, 100, FULL_STEP, SANE_FALSE, SANE_TRUE,  63, 120, 8736,   601, 192, 4905,  337, 0.30, 0.4, 192},
+  {MOTOR_HP2400, 100, SANE_FALSE, 100, HALF_STEP, SANE_FALSE, SANE_TRUE,  63, 120, 8736,   601, 192, 4905,  337, 0.30, 0.4, 192},
+  {MOTOR_HP2400, 150, SANE_FALSE, 150, HALF_STEP, SANE_FALSE, SANE_TRUE,  63, 120, 8736,   601, 192, 4905,  337, 0.30, 0.4, 192},
   {MOTOR_HP2400, 300, SANE_FALSE, 300, HALF_STEP, SANE_FALSE, SANE_TRUE , 63, 32, 16703,  2188, 192, 4905,  337, 0.30, 0.4, 192},
   {MOTOR_HP2400, 600, SANE_FALSE, 600, FULL_STEP, SANE_FALSE, SANE_TRUE , 63,  3, 18761, 18761, 192, 4905,  337, 0.30, 0.4, 192},
+  {MOTOR_HP2400,1200, SANE_FALSE,1200, HALF_STEP, SANE_FALSE, SANE_TRUE , 63,  3, 18761, 18761, 192, 4905,  337, 0.30, 0.4, 192},
 
   /* XP 200 motor settings */
   {MOTOR_XP200,  75,  SANE_TRUE,  75, HALF_STEP, SANE_TRUE , SANE_FALSE, 0, 4,  6000,  2136, 8, 12000, 1200, 0.3, 0.5, 1},
