@@ -72,43 +72,42 @@
     if (status != SANE_STATUS_GOOD) return status;      \
   } while (SANE_FALSE)
 
+#define DBGSTART DBG (DBG_proc, "%s start\n", __FUNCTION__);
+#define DBGCOMPLETED DBG (DBG_proc, "%s completed\n", __FUNCTION__);
 
 /* Flags */
-#define GENESYS_FLAG_UNTESTED     (1 << 0)	/* Print a warning for these scanners */
-#define GENESYS_FLAG_14BIT_GAMMA  (1 << 1)	/* use 14bit Gamma table instead of 12 */
-#define GENESYS_FLAG_LAZY_INIT    (1 << 2)	/* skip extensive ASIC test at init   */
+#define GENESYS_FLAG_UNTESTED     (1 << 0)	/**> Print a warning for these scanners */
+#define GENESYS_FLAG_14BIT_GAMMA  (1 << 1)	/**> use 14bit Gamma table instead of 12 */
+#define GENESYS_FLAG_LAZY_INIT    (1 << 2)	/**> skip extensive ASIC test at init   */
 #define GENESYS_FLAG_XPA          (1 << 3)
-#define GENESYS_FLAG_SKIP_WARMUP  (1 << 4)	/* skip genesys_warmup()              */
-#define GENESYS_FLAG_OFFSET_CALIBRATION   (1 << 5)	/* do offset calibration      */
-#define GENESYS_FLAG_SEARCH_START (1 << 6)	/* do start search beofre scanning    */
-#define GENESYS_FLAG_REPARK       (1 << 7)	/* repark head (and check for lock) by 
+#define GENESYS_FLAG_SKIP_WARMUP  (1 << 4)	/**> skip genesys_warmup()              */
+#define GENESYS_FLAG_OFFSET_CALIBRATION   (1 << 5)	/**> do offset calibration      */
+#define GENESYS_FLAG_SEARCH_START (1 << 6)	/**> do start search before scanning    */
+#define GENESYS_FLAG_REPARK       (1 << 7)	/**> repark head (and check for lock) by 
 						   moving without scanning */
-#define GENESYS_FLAG_DARK_CALIBRATION (1 << 8)	/* do dark calibration */
-#define GENESYS_FLAG_STAGGERED_LINE   (1 << 9)	/* pixel columns are shifted vertically for hi-res modes */
+#define GENESYS_FLAG_DARK_CALIBRATION (1 << 8)	/**> do dark calibration */
+#define GENESYS_FLAG_STAGGERED_LINE   (1 << 9)	/**> pixel columns are shifted vertically for hi-res modes */
 
-#define GENESYS_FLAG_MUST_WAIT        (1 << 10)	/* tells wether the scanner 
-						   should wait 1 minute after 
-						   init before doing anything 
-						*/
+#define GENESYS_FLAG_MUST_WAIT        (1 << 10)	/**> tells wether the scanner must wait for the head when parking */
 
 
-#define GENESYS_FLAG_ALT_SLOPE_CREATE (1 << 11)	/* use alternative slope
-						   creation function */
+#define GENESYS_FLAG_ALT_SLOPE_CREATE (1 << 11)	/**> use alternative slope creation function */
 
-#define GENESYS_FLAG_DARK_WHITE_CALIBRATION (1 << 12) /* yet another calibration method. does white and dark shading in one run, depending on a black and a white strip*/
-#define GENESYS_FLAG_CUSTOM_GAMMA     (1 << 13)       /* allow custom gamma tables */
-#define GENESYS_FLAG_NO_CALIBRATION   (1 << 14)       /* allow scanners to use skip the calibration, needed for sheetfed scanners */
-#define GENESYS_FLAG_HALF_CCD_MODE    (1 << 15)       /* scanner has setting for half ccd mode */
+#define GENESYS_FLAG_DARK_WHITE_CALIBRATION (1 << 12) /**> yet another calibration method. does white and dark shading in one run, depending on a black and a white strip*/
+#define GENESYS_FLAG_CUSTOM_GAMMA     (1 << 13)       /**> allow custom gamma tables */
+#define GENESYS_FLAG_NO_CALIBRATION   (1 << 14)       /**> allow scanners to use skip the calibration, needed for sheetfed scanners */
+#define GENESYS_FLAG_HALF_CCD_MODE    (1 << 15)       /**> scanner has setting for half ccd mode */
+#define GENESYS_FLAG_ODD_EVEN_CIS     (1 << 16)       /**> scan odd and even pixels come in separated lines */
 
-#define GENESYS_HAS_NO_BUTTONS       0              /* scanner has no supported button */
-#define GENESYS_HAS_SCAN_SW          (1 << 0)       /* scanner has SCAN button */
-#define GENESYS_HAS_FILE_SW          (1 << 1)       /* scanner has FILE button */
-#define GENESYS_HAS_COPY_SW          (1 << 2)       /* scanner has COPY button */
-#define GENESYS_HAS_EMAIL_SW         (1 << 3)       /* scanner has EMAIL button */
-#define GENESYS_HAS_PAGE_LOADED_SW   (1 << 4)       /* scanner has paper in detection */
-#define GENESYS_HAS_OCR_SW           (1 << 5)       /* scanner has OCR button */
-#define GENESYS_HAS_POWER_SW         (1 << 6)       /* scanner has power button */
-#define GENESYS_HAS_CALIBRATE        (1 << 7)       /* scanner has 'calibrate' software button to start calibration */
+#define GENESYS_HAS_NO_BUTTONS       0              /**> scanner has no supported button */
+#define GENESYS_HAS_SCAN_SW          (1 << 0)       /**> scanner has SCAN button */
+#define GENESYS_HAS_FILE_SW          (1 << 1)       /**> scanner has FILE button */
+#define GENESYS_HAS_COPY_SW          (1 << 2)       /**> scanner has COPY button */
+#define GENESYS_HAS_EMAIL_SW         (1 << 3)       /**> scanner has EMAIL button */
+#define GENESYS_HAS_PAGE_LOADED_SW   (1 << 4)       /**> scanner has paper in detection */
+#define GENESYS_HAS_OCR_SW           (1 << 5)       /**> scanner has OCR button */
+#define GENESYS_HAS_POWER_SW         (1 << 6)       /**> scanner has power button */
+#define GENESYS_HAS_CALIBRATE        (1 << 7)       /**> scanner has 'calibrate' software button to start calibration */
 
 /* USB control message values */
 #define REQUEST_TYPE_IN		(USB_TYPE_VENDOR | USB_DIR_IN)
@@ -123,11 +122,12 @@
 #define GPIO_OUTPUT_ENABLE	0x89
 #define GPIO_READ		0x8a
 #define GPIO_WRITE		0x8b
+#define VALUE_BUF_ENDACCESS	0x8c
+#define VALUE_GET_REGISTER	0x8e
 #define INDEX			0x00
 
 /* todo: used?
 #define VALUE_READ_STATUS	0x86
-#define VALUE_BUF_ENDACCESS	0x8C
 */
 
 /* Read/write bulk data/registers */
@@ -146,19 +146,24 @@
 #define AFE_SET        2
 #define AFE_POWER_SAVE 4
 
-#define LOWORD(x)  ((uint16_t)(x & 0xffff))
-#define HIWORD(x)  ((uint16_t)(x >> 16))
+#define LOWORD(x)  ((uint16_t)((x) & 0xffff))
+#define HIWORD(x)  ((uint16_t)((x) >> 16))
 #define LOBYTE(x)  ((uint8_t)((x) & 0xFF))
 #define HIBYTE(x)  ((uint8_t)((x) >> 8))
 
 /* Global constants */
-/* todo: check if those are the same for every scanner */
-#define SYSTEM_CLOCK		32	/* todo: ? */
+/* TODO: emove this leftover of early backend days */
 #define MOTOR_SPEED_MAX		350
-													      /*#define MOTOR_GEAR    *//*600 1200 * todo: base y res? --> model */
-#define PIXEL_TIME		((double) 24 / SYSTEM_CLOCK)
 #define DARK_VALUE		0
 
+#define PWRBIT	        0x80
+#define BUFEMPTY	0x40
+#define FEEDFSH	        0x20
+#define SCANFSH	        0x10
+#define HOMESNR	        0x08
+#define LAMPSTS	        0x04
+#define FEBUSY	        0x02
+#define MOTORENB	0x01
 
 typedef struct
 {
@@ -234,15 +239,19 @@ typedef enum Genesys_Color_Order
 Genesys_Color_Order;
 
 
-#define MAX_SCANNERS 30
+#define MAX_SCANNERS 50
 #define MAX_RESOLUTIONS 13
 #define MAX_DPI 4
 
 #define GENESYS_GL646	 646
 #define GENESYS_GL841	 841
+#define GENESYS_GL843	 843
+#define GENESYS_GL846	 846
+#define GENESYS_GL847	 847
+#define GENESYS_GL848	 848
+#define GENESYS_GL124	 124
 
-/*135 registers for gl841 + 1 null-reg*/
-#define GENESYS_MAX_REGS 136
+#define GENESYS_MAX_REGS 256
 
 #define DAC_WOLFSON_UMAX   0
 #define DAC_WOLFSON_ST12   1
@@ -255,6 +264,11 @@ Genesys_Color_Order;
 #define DAC_WOLFSON_XP300  8
 #define DAC_WOLFSON_HP3670 9
 #define DAC_WOLFSON_DSM600 10
+#define DAC_CANONLIDE200   11
+#define DAC_KVSS080        12
+#define DAC_G4050          13
+#define DAC_CANONLIDE110   14
+#define DAC_PLUSTEK_3600   15
 
 #define CCD_UMAX         0
 #define CCD_ST12         1	/* SONY ILX548: 5340 Pixel  ??? */
@@ -271,6 +285,12 @@ Genesys_Color_Order;
 #define CCD_DSMOBILE600  12
 #define CCD_XP300        13
 #define CCD_DP685        14
+#define CIS_CANONLIDE200 15
+#define CIS_CANONLIDE100 16
+#define CCD_KVSS080      17
+#define CCD_G4050        18
+#define CIS_CANONLIDE110 19
+#define CCD_PLUSTEK_3600 20
 
 #define GPO_UMAX         0
 #define GPO_ST12         1
@@ -284,6 +304,11 @@ Genesys_Color_Order;
 #define GPO_HP3670       9
 #define GPO_DP665        10
 #define GPO_DP685        11
+#define GPO_CANONLIDE200 12
+#define GPO_KVSS080      13
+#define GPO_G4050        14
+#define GPO_CANONLIDE110 15
+#define GPO_PLUSTEK_3600 16
 
 #define MOTOR_UMAX       0
 #define MOTOR_5345       1
@@ -297,6 +322,12 @@ Genesys_Color_Order;
 #define MOTOR_DP665      10
 #define MOTOR_ROADWARRIOR 11
 #define MOTOR_DSMOBILE_600 12
+#define MOTOR_CANONLIDE200 13
+#define MOTOR_CANONLIDE100 14
+#define MOTOR_KVSS080      15
+#define MOTOR_G4050        16
+#define MOTOR_CANONLIDE110 17
+#define MOTOR_PLUSTEK_3600 18
 
 
 /* Forward typedefs */
@@ -416,6 +447,12 @@ typedef struct Genesys_Command_Set
      * move scanning head to transparency adapter
      */
     SANE_Status (*move_to_ta) (Genesys_Device * dev);
+
+    /**
+     * write shading data calibration to ASIC
+     */
+    SANE_Status (*send_shading_data) (Genesys_Device * dev, uint8_t * data, int size);
+
 } Genesys_Command_Set;
 
 typedef struct Genesys_Model
@@ -515,6 +552,10 @@ typedef struct
   /**> Disable interpolation for xres<yres*/
   int disable_interpolation;
 
+  /**> Use double x resolution internally to provide better
+   * quality */
+  int double_xres;
+
   /**> true is lineart is generated from gray data by
    * the dynamic rasterization algo */
   int dynamic_lineart;
@@ -563,12 +604,13 @@ struct Genesys_Calibration_Cache
 struct Genesys_Device
 {
   SANE_Int dn;
+  SANE_Bool usb_mode;
   SANE_String file_name;
   SANE_String calib_file;
   Genesys_Model *model;
 
-  Genesys_Register_Set reg[GENESYS_MAX_REGS];
-  Genesys_Register_Set calib_reg[GENESYS_MAX_REGS];
+  Genesys_Register_Set reg[256];
+  Genesys_Register_Set calib_reg[256];
   Genesys_Settings settings;
   Genesys_Frontend frontend;
   Genesys_Sensor sensor;
@@ -591,6 +633,7 @@ struct Genesys_Device
   SANE_Int lamp_off_time;
 
   SANE_Bool read_active;
+  SANE_Bool parking;		/**> signal wether the park command has been issued */
   SANE_Bool document;		/**> for sheetfed scanner's, is TRUE when there
 				   is a document in the scanner */
 
@@ -613,6 +656,20 @@ struct Genesys_Device
   Genesys_Calibration_Cache *calibration_cache;
 
   struct Genesys_Device *next;
+
+  int segnb;       /**> number of segments composing the sensor */
+  int line_interp; /**> number of lines used in line interpolation */
+  int line_count;  /**> number of scan lines used during scan */
+  size_t bpl;      /**> bytes per full scan widthline */
+  size_t skip;     /**> bytes to skip from start of line to get first required pixel */
+  size_t dist;     /**> bytes distance between an odd and an even pixel */
+  size_t len;      /**> number of even pixels */
+  size_t cur;      /**> current pixel position within sub window */
+  Genesys_Buffer oe_buffer; /**> buffer to handle even/odd data */
+
+  SANE_Bool buffer_image; /**> when true the scanned picture is first buffered
+			   * to allow software image enhancements */
+  SANE_Byte *img_buffer; /**> image buffer where the scanned picture is stored */
 };
 
 typedef struct Genesys_USB_Device_Entry
@@ -637,6 +694,8 @@ extern void
 sanei_genesys_set_reg_from_set (Genesys_Register_Set * regs,
 				SANE_Byte address, SANE_Byte value);
 
+extern SANE_Status sanei_genesys_init_cmd_set (Genesys_Device * dev);
+
 extern SANE_Status
 sanei_genesys_read_register (Genesys_Device * dev, uint8_t reg,
 			     uint8_t * val);
@@ -646,7 +705,21 @@ sanei_genesys_write_register (Genesys_Device * dev, uint8_t reg,
 			      uint8_t val);
 
 extern SANE_Status
-sanei_genesys_get_status (Genesys_Device * dev, uint8_t * status);
+sanei_genesys_read_hregister (Genesys_Device * dev, uint8_t reg,
+			     uint8_t * val);
+
+extern SANE_Status
+sanei_genesys_write_hregister (Genesys_Device * dev, uint8_t reg,
+			      uint8_t val);
+
+extern SANE_Status sanei_genesys_write_0x8c (Genesys_Device * dev, uint8_t index, uint8_t val);
+
+extern SANE_Status sanei_genesys_get_status (Genesys_Device * dev, uint8_t * status);
+
+extern void sanei_genesys_print_status (uint8_t val);
+
+extern SANE_Status
+sanei_genesys_write_ahb (SANE_Int dn, uint32_t addr, uint32_t size, uint8_t * data);
 
 extern void sanei_genesys_init_fe (Genesys_Device * dev);
 
@@ -684,6 +757,10 @@ sanei_genesys_calculate_zmode (Genesys_Device * dev,
 extern SANE_Status
 sanei_genesys_set_buffer_address (Genesys_Device * dev, uint32_t addr);
 
+extern SANE_Status
+sanei_genesys_fe_read_data (Genesys_Device * dev, uint8_t addr,
+			    uint16_t *data);
+
 extern SANE_Status
 sanei_genesys_fe_write_data (Genesys_Device * dev, uint8_t addr,
 			     uint16_t data);
@@ -763,11 +840,28 @@ sanei_genesys_buffer_produce(Genesys_Buffer * buf, size_t size);
 extern SANE_Status
 sanei_genesys_buffer_consume(Genesys_Buffer * buf, size_t size);
 
+extern SANE_Status
+sanei_genesys_set_double(Genesys_Register_Set *regs, SANE_Byte addr, uint16_t value);
+
+extern SANE_Status
+sanei_genesys_set_triple(Genesys_Register_Set *regs, SANE_Byte addr, uint32_t value);
+
+extern SANE_Status
+sanei_genesys_get_double(Genesys_Register_Set *regs, SANE_Byte addr, uint16_t *value);
+
+extern SANE_Status
+sanei_genesys_get_triple(Genesys_Register_Set *regs, SANE_Byte addr, uint32_t *value);
+
+extern SANE_Status
+sanei_genesys_wait_for_home(Genesys_Device *dev);
 
 /*---------------------------------------------------------------------------*/
 /*                ASIC specific functions declarations                       */
 /*---------------------------------------------------------------------------*/
 extern SANE_Status sanei_gl646_init_cmd_set (Genesys_Device * dev);
 extern SANE_Status sanei_gl841_init_cmd_set (Genesys_Device * dev);
+extern SANE_Status sanei_gl843_init_cmd_set (Genesys_Device * dev);
+extern SANE_Status sanei_gl847_init_cmd_set (Genesys_Device * dev);
+extern SANE_Status sanei_gl124_init_cmd_set (Genesys_Device * dev);
 
 #endif /* not GENESYS_LOW_H */
