@@ -73,7 +73,7 @@
 
 /* ------------------------------------------------------------------------- */
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -91,12 +91,12 @@
 # include <libc.h>		/* NeXTStep/OpenStep */
 #endif
 
-#include "sane/sane.h"
-#include "sane/sanei_usb.h"
-#include "sane/saneopts.h"
-#include "sane/sanei_config.h"
-#include "sane/sanei_thread.h"
-#include "sane/sanei_backend.h"
+#include "../include/sane/sane.h"
+#include "../include/sane/sanei_usb.h"
+#include "../include/sane/saneopts.h"
+#include "../include/sane/sanei_config.h"
+#include "../include/sane/sanei_thread.h"
+#include "../include/sane/sanei_backend.h"
 
 #define RTCMD_GETREG		0x80
 #define	RTCMD_READSRAM		0x81
@@ -117,7 +117,6 @@
 typedef int (*rts8801_callback) (void *param, unsigned bytes, void *data);
 
 #define DEBUG 1
-#define MM_PER_INCH 25.4
 #define SCANNER_UNIT_TO_FIXED_MM(number) SANE_FIX(number * MM_PER_INCH / 1200)
 #define FIXED_MM_TO_SCANNER_UNIT(number) SANE_UNFIX(number) * 1200 / MM_PER_INCH
 
@@ -164,7 +163,7 @@ struct hp3500_data
   int sfd;
   int pipe_r;
   int pipe_w;
-  int reader_pid;
+  SANE_Pid reader_pid;
 
   int resolution;
   int mode;
@@ -260,7 +259,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   sanei_thread_init ();
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
   sanei_usb_find_devices (0x03f0, 0x2205, attachScanner);
   sanei_usb_find_devices (0x03f0, 0x2005, attachScanner);
@@ -3393,28 +3392,29 @@ reader_process (void *pv)
   sigset_t sigterm_set;
   struct SIGACTION act;
   struct hp3500_write_info winfo;
+  int status;
 
   if (sanei_thread_is_forked ())
     {
       close (scanner->pipe_r);
+
+      sigfillset (&ignore_set);
+      sigdelset (&ignore_set, SIGTERM);
+#if     defined (__APPLE__) && defined (__MACH__)
+      sigdelset (&ignore_set, SIGUSR2);
+#endif
+      sigprocmask (SIG_SETMASK, &ignore_set, 0);
+
+      sigemptyset (&sigterm_set);
+      sigaddset (&sigterm_set, SIGTERM);
+
+      memset (&act, 0, sizeof (act));
+#ifdef     _POSIX_SOURCE
+      act.sa_handler = sigtermHandler;
+#endif
+      sigaction (SIGTERM, &act, 0);
     }
 
-  sigfillset (&ignore_set);
-  sigdelset (&ignore_set, SIGTERM);
-#if defined (__APPLE__) && defined (__MACH__)
-  sigdelset (&ignore_set, SIGUSR2);
-#endif
-  sigprocmask (SIG_SETMASK, &ignore_set, 0);
-
-  sigemptyset (&sigterm_set);
-  sigaddset (&sigterm_set, SIGTERM);
-
-  memset (&act, 0, sizeof (act));
-#ifdef _POSIX_SOURCE
-  act.sa_handler = sigtermHandler;
-#endif
-  sigaction (SIGTERM, &act, 0);
-
 
   /* Warm up the lamp again if our last scan ended more than 5 minutes ago. */
   time (&t);
@@ -3448,6 +3448,8 @@ reader_process (void *pv)
        scanner->actres_pixels.bottom - scanner->actres_pixels.top,
        scanner->resolution, scanner->mode, (rts8801_callback) writefunc,
        &winfo) >= 0)
-    exit (SANE_STATUS_GOOD);
-  exit (SANE_STATUS_IO_ERROR);
+    status = SANE_STATUS_GOOD;
+  status = SANE_STATUS_IO_ERROR;
+  close (scanner->pipe_w);
+  return status;
 }
