@@ -1,5 +1,6 @@
 /* SANE - Scanner Access Now Easy.
 
+   Copyright (C) 2007-2008 Nicolas Martin, <nicols-guest at alioth dot debian dot org>
    Copyright (C) 2006-2007 Wittawat Yamwong <wittawat@web.de>
 
    This file is part of the SANE package.
@@ -97,18 +98,22 @@
 # include <stdint.h>		/* available in ISO C99 */
 #else
 # include <sys/types.h>
-typedef u_int8_t uint8_t;
-typedef u_int16_t uint16_t;
-typedef u_int32_t uint32_t;
+typedef uint8_t uint8_t;
+typedef uint16_t uint16_t;
+typedef uint32_t uint32_t;
 #endif /* HAVE_STDINT_H */
 
+#ifdef HAVE_INTTYPES_H 
+# include <inttypes.h>          /* available in ISO C99 */
+#endif /* HAVE_INTTYPES_H */
+
 /** \addtogroup API
  *  @{ */
 /** \name Version of the driver */
 /**@{*/
 #define PIXMA_VERSION_MAJOR 0
-#define PIXMA_VERSION_MINOR 13
-#define PIXMA_VERSION_BUILD 1
+#define PIXMA_VERSION_MINOR 16
+#define PIXMA_VERSION_BUILD 2
 /**@}*/
 
 /** \name Error codes */
@@ -126,6 +131,7 @@ typedef u_int32_t uint32_t;
 #define PIXMA_EPAPER_JAMMED     -11
 #define PIXMA_ECOVER_OPEN       -12
 #define PIXMA_ENO_PAPER         -13
+#define PIXMA_EOF               -14
 /**@}*/
 
 /** \name Capabilities for using with pixma_config_t::cap */
@@ -140,6 +146,7 @@ typedef u_int32_t uint32_t;
 #define PIXMA_CAP_ADFDUP       ((1 << 7) | PIXMA_CAP_ADF)
 #define PIXMA_CAP_CIS          (0)
 #define PIXMA_CAP_CCD          (1 << 8)
+#define PIXMA_CAP_LINEART      (1 << 9)
 #define PIXMA_CAP_EXPERIMENT   (1 << 31)
 /**@}*/
 
@@ -152,6 +159,8 @@ typedef u_int32_t uint32_t;
 /**@}*/
 /** @} end of API group */
 
+#define PIXMA_CONFIG_FILE "pixma.conf"
+#define MAX_CONF_DEVICES 15
 
 struct pixma_t;
 struct pixma_scan_ops_t;
@@ -240,7 +249,7 @@ struct pixma_scan_param_t
     /** Size in bytes of the whole image.
      *  image_size = line_size * h <br>
      *  This field will be set by pixma_check_scan_param(). */
-  unsigned image_size;
+  uint64_t image_size;
 
     /** Channels per pixel. 1 = grayscale, 3 = color */
   unsigned channels;
@@ -255,9 +264,12 @@ struct pixma_scan_param_t
 
   /*! \name Scan area in pixels
    * (0,0) = top left; positive x extends to the right; positive y to the
-   *  bottom; in pixels. */
+   *  bottom; in pixels.
+   * xs is the offset in x direction of the selected scan range relative
+   * to the range read from the scanner and wx the width in x direction
+   * of the scan line read from scanner. */
   /*@{ */
-  unsigned x, y, w, h;
+  unsigned x, y, w, h,   xs, wx;
   /*@} */
 
     /** Gamma table. 4096 entries, 12 bit => 8 bit. If \c NULL, default gamma
@@ -266,6 +278,9 @@ struct pixma_scan_param_t
 
     /** \see #pixma_paper_source_t */
   pixma_paper_source_t source;
+
+    /** The current page # in the same ADF scan session, 0 in non ADF */
+  unsigned adf_pageid;
 };
 
 /** PIXMA model information */
@@ -274,15 +289,16 @@ struct pixma_config_t
   /* If you change this structure, don't forget to update the device list in
    * subdrivers. */
   const char *name;	   /**< Model name. */
-  uint16_t vid;		   /**< USB Vendor ID */
-  uint16_t pid;		   /**< USB Product ID */
-  unsigned iface;	   /**< USB Interface number */
+  const char *model;   /**< Short model */
+  uint16_t vid;		     /**< USB Vendor ID */
+  uint16_t pid;		     /**< USB Product ID */
+  unsigned iface;	     /**< USB Interface number */
   const pixma_scan_ops_t *ops;	  /**< Subdriver ops */
-  unsigned xdpi;	   /**< Maximum horizontal resolution[DPI] */
-  unsigned ydpi;	   /**< Maximum vertical resolution[DPI] */
-  unsigned width;	   /**< Maximum width of scannable area in pixels at 75DPI */
+  unsigned xdpi;	     /**< Maximum horizontal resolution[DPI] */
+  unsigned ydpi;	     /**< Maximum vertical resolution[DPI] */
+  unsigned width;	     /**< Maximum width of scannable area in pixels at 75DPI */
   unsigned height;	   /**< Maximum height of scannable area in pixels at 75DPI */
-  unsigned cap;		   /**< Capability bitfield \see PIXMA_CAP_* */
+  unsigned cap;		     /**< Capability bitfield \see PIXMA_CAP_* */
 };
 
 
@@ -311,7 +327,7 @@ void pixma_set_debug_level (int level);
  *
  *  \return The number of scanners found currently. The return value is
  *  guaranteed to be valid until the next call to pixma_find_scanners(). */
-int pixma_find_scanners (void);
+int pixma_find_scanners (const char **conf_devices);
 
 /** Return the model name of the device \a devnr. */
 const char *pixma_get_device_model (unsigned devnr);
@@ -380,6 +396,14 @@ int pixma_check_scan_param (pixma_t *, pixma_scan_param_t *);
  */
 uint32_t pixma_wait_event (pixma_t *, int timeout);
 
+/** Activate connection to scanner */
+int pixma_activate_connection (pixma_t *);
+
+/** De-activate connection to scanner */
+
+int pixma_deactivate_connection (pixma_t *);
+
+
 /** Enable or disable background tasks. Currently, the only one task
  *  is submitting interrupt URB in background.
  *  \param[in] enabled if not zero, enable background task.
