@@ -76,12 +76,12 @@
 #define D480_PID   0x26ed
 #define MF4320_PID 0x26ee
 #define MF3200_PID 0x2684
+#define MF6500_PID 0x2686
 /* the following are all untested */
 #define MF5630_PID 0x264e
 #define MF5650_PID 0x264f
 #define MF8100_PID 0x2659
-#define MF6500_PID 0x2686
-
+#define MF8030_PID 0x2707
 
 
 enum iclass_state_t
@@ -180,6 +180,7 @@ activate (pixma_t * s, uint8_t x)
     {
     case MF4200_PID:
     case MF4600_PID:
+    case MF6500_PID:
     case D480_PID:
     case MF4360_PID:
       return iclass_exec (s, &mf->cb, 1);
@@ -209,6 +210,7 @@ select_source (pixma_t * s)
     {
     case MF4200_PID:
     case MF4600_PID:
+    case MF6500_PID:
     case D480_PID:
     case MF4360_PID:
       return iclass_exec (s, &mf->cb, 0);
@@ -241,6 +243,7 @@ send_scan_param (pixma_t * s)
     {
     case MF4200_PID:
     case MF4600_PID:
+    case MF6500_PID:
     case D480_PID:
     case MF4360_PID:
       return iclass_exec (s, &mf->cb, 0);
@@ -264,7 +267,9 @@ request_image_block (pixma_t * s, unsigned flag, uint8_t * info,
   pixma_set_be16 (cmd_read_image, mf->cb.buf);
   mf->cb.buf[8] = flag;
   mf->cb.buf[10] = 0x06;
-  expected_len = (s->cfg->pid == MF4600_PID) ? 512 : hlen;
+  expected_len = (s->cfg->pid == MF4600_PID ||
+                  s->cfg->pid == MF6500_PID ||
+                  s->cfg->pid == MF8030_PID) ? 512 : hlen;
   mf->cb.reslen = pixma_cmd_transaction (s, mf->cb.buf, 11, mf->cb.buf, expected_len);
   if (mf->cb.reslen >= hlen)
     {
@@ -272,7 +277,9 @@ request_image_block (pixma_t * s, unsigned flag, uint8_t * info,
       *size = pixma_get_be16 (mf->cb.buf + 6);    /* 16bit size */
       error = 0;
 
-      if (s->cfg->pid == MF4600_PID)
+      if (s->cfg->pid == MF4600_PID ||
+          s->cfg->pid == MF6500_PID ||
+          s->cfg->pid == MF8030_PID)
         {                                         /* 32bit size */
           *datalen = mf->cb.reslen - hlen;
           *size = (*datalen + hlen == 512) ? pixma_get_be32 (mf->cb.buf + 4) - *datalen : 0;
@@ -292,7 +299,9 @@ read_image_block (pixma_t * s, uint8_t * data, unsigned size)
   int error;
   unsigned maxchunksize, chunksize, count = 0;
   
-  maxchunksize = MAX_CHUNK_SIZE * ((s->cfg->pid == MF4600_PID) ? 4 : 1);
+  maxchunksize = MAX_CHUNK_SIZE * ((s->cfg->pid == MF4600_PID || 
+                                    s->cfg->pid == MF6500_PID ||
+                                    s->cfg->pid == MF8030_PID) ? 4 : 1);
   while (size)
     {
       if (size >= maxchunksize)
@@ -324,6 +333,7 @@ read_error_info (pixma_t * s, void *buf, unsigned size)
     {
     case MF4200_PID:
     case MF4600_PID:
+    case MF6500_PID:
     case D480_PID:
     case MF4360_PID:
       error = iclass_exec (s, &mf->cb, 0);
@@ -585,9 +595,11 @@ iclass_fill_buffer (pixma_t * s, pixma_imagebuf_t * ib)
       if (n != 0)
         {
           if (s->param->channels != 1 &&
-	      s->cfg->pid != MF4600_PID)
+	          s->cfg->pid != MF4600_PID &&
+	          s->cfg->pid != MF6500_PID &&
+	          s->cfg->pid != MF8030_PID)
             {
-              /* color and not MF46xx */
+              /* color and not MF46xx or MF65xx */
               pack_rgb (mf->blkptr, n, mf->raw_width, mf->lineptr);
             }
           else
@@ -696,10 +708,11 @@ const pixma_config_t pixma_iclass_devices[] = {
   DEV ("Canon imageCLASS MF4320", "MF4320", MF4320_PID, 600, 640, 877, PIXMA_CAP_ADF),
   DEV ("Canon imageCLASS MF4010", "MF4010", MF4010_PID, 600, 640, 877, 0),
   DEV ("Canon imageCLASS MF3240", "MF3240", MF3200_PID, 600, 640, 877, 0),
+  DEV ("Canon imageClass MF6500", "MF6500", MF6500_PID, 600, 640, 877, PIXMA_CAP_ADF),
   /* FIXME: the following capabilities all need updating/verifying */
   DEV ("Canon imageCLASS MF5630", "MF5630", MF5630_PID, 600, 640, 877, PIXMA_CAP_ADF),
   DEV ("Canon laserBase MF5650", "MF5650", MF5650_PID, 600, 640, 877, PIXMA_CAP_ADF),
   DEV ("Canon imageCLASS MF8170c", "MF8170c", MF8100_PID, 600, 640, 877, PIXMA_CAP_ADF),
-  DEV ("Canon imageClass MF6500", "MF6500", MF6500_PID, 600, 640, 877, PIXMA_CAP_ADF),
+  DEV ("Canon imageClass MF8030", "MF8030", MF8030_PID, 600, 640, 877, PIXMA_CAP_ADF),  
   DEV (NULL, NULL, 0, 0, 0, 0, 0)
 };
