@@ -157,6 +157,11 @@
 #define MP640_PID 0x173f
 #define MP990_PID 0x1740
 
+/* PIXMA 2010 vintage */
+#define MX340_PID 0x1741
+#define MX350_PID 0x1742
+#define MX870_PID 0x1743
+
 /* Generation 4 XML messages that encapsulates the Pixma protocol messages */
 #define XML_START_1   \
 "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\
@@ -534,11 +539,11 @@ calc_raw_width (const mp150_t * mp, const pixma_scan_param_t * param)
     }
   else if (param->channels == 1)
     {
-      raw_width = ALIGN_SUP (param->w + param->xs, 12);					/******** Changed here *******/
+      raw_width = ALIGN_SUP (param->w + param->xs, 12);
     }
   else
     {
-      raw_width = ALIGN_SUP (param->w + param->xs, 4);					/******** Changed here *******/
+      raw_width = ALIGN_SUP (param->w + param->xs, 4);
     }
   return raw_width;
 }
@@ -585,8 +590,12 @@ calc_shifting (pixma_t * s)
 
       case MP800_PID:
       case MP800R_PID:
-      case MP830_PID:
       case MP960_PID:
+        if (s->param->xdpi == 2400)
+          mp->stripe_shift = 6;
+        break;
+
+      case MP830_PID:
       case MP810_PID:
         if (s->param->xdpi == 2400) 
           mp->stripe_shift = 3;
@@ -610,7 +619,7 @@ calc_shifting (pixma_t * s)
             break;
 
           case MP980_PID:
-	  case MP990_PID:
+          case MP990_PID:
             if (s->param->ydpi > 150)
               {
                 mp->color_shift = s->param->ydpi / 75;
@@ -671,6 +680,8 @@ send_scan_param (pixma_t * s)
       pixma_set_be16 (s->param->xdpi | 0x8000, data + 0x04);
       pixma_set_be16 (s->param->ydpi | 0x8000, data + 0x06);
       pixma_set_be32 (s->param->x, data + 0x08);
+      if (mp->generation == 2)
+        pixma_set_be32 (s->param->x - s->param->xs, data + 0x08);
       pixma_set_be32 (s->param->y, data + 0x0c);
       pixma_set_be32 (raw_width, data + 0x10);
       pixma_set_be32 (h, data + 0x14);
@@ -1092,14 +1103,14 @@ post_process_image_data (pixma_t * s, pixma_imagebuf_t * ib)
               && s->param->xdpi == 4800)
                 mp970_reorder_pixels (mp->linebuf, sptr, c, s->param->wx, line_size);
 
-	  /* Crop line to selected borders */
-	  memcpy(cptr, sptr + cx, cw);
+          /* Crop line to selected borders */
+          memmove(cptr, sptr + cx, cw);
 	  
           /* Color to Grayscale convert for CCD sensor */
           if (is_ccd_grayscale (s))
-              cptr = gptr = rgb_to_gray (gptr, cptr, s->param->w, c);				/******** Changed here *******/
-	  else
-	      cptr += cw;
+              cptr = gptr = rgb_to_gray (gptr, cptr, s->param->w, c);
+          else
+              cptr += cw;
         }
     }
   ib->rptr = mp->imgbuf;
@@ -1212,8 +1223,11 @@ mp150_check_param (pixma_t * s, pixma_scan_param_t * sp)
   /* Those devices can scan up to 14" with ADF, but A4 11.7" in flatbed */
   if (( s->cfg->pid == MX850_PID ||
         s->cfg->pid == MX860_PID ||
+        s->cfg->pid == MX870_PID ||
         s->cfg->pid == MX320_PID ||
         s->cfg->pid == MX330_PID ||
+        s->cfg->pid == MX340_PID ||
+        s->cfg->pid == MX350_PID ||
         s->cfg->pid == MX7600_PID )
        &&
         sp->source == PIXMA_SOURCE_FLATBED)
@@ -1246,6 +1260,24 @@ mp150_check_param (pixma_t * s, pixma_scan_param_t * sp)
       sp->xdpi *= k;
       sp->ydpi = sp->xdpi;
     }
+
+  if (sp->source == PIXMA_SOURCE_ADF || sp->source == PIXMA_SOURCE_ADFDUP)
+    {
+      uint8_t k = 1;
+
+  /* ADF/ADF duplex mode: max scan res is 600 dpi, at least for generation 4 */
+      if (mp->generation >= 4)
+        k = sp->xdpi / MIN (sp->xdpi, 600);
+      sp->x /= k;
+      sp->xs /= k;
+      sp->y /= k;
+      sp->w /= k;
+      sp->wx /= k;
+      sp->h /= k;
+      sp->xdpi /= k;
+      sp->ydpi = sp->xdpi;
+    }
+
   return 0;
 }
 
@@ -1605,6 +1637,11 @@ const pixma_config_t pixma_mp150_devices[] = {
  * Not sure if anything's going wrong here, leaving as is
   DEVICE ("Canon PIXMA MX860", "MX860", MX860_PID, 2400, 638, 880, PIXMA_CAP_CIS | PIXMA_CAP_ADFDUP),*/
 
+  /* PIXMA 2010 vintage */
+  DEVICE ("Canon PIXMA MX340", "MX340", MX340_PID, 1200, 638, 1050, PIXMA_CAP_CIS | PIXMA_CAP_ADF),
+  DEVICE ("Canon PIXMA MX350", "MX350", MX350_PID, 1200, 638, 1050, PIXMA_CAP_CIS | PIXMA_CAP_ADF),
+  DEVICE ("Canon PIXMA MX870", "MX870", MX870_PID, 2400, 638, 1050, PIXMA_CAP_CIS | PIXMA_CAP_ADFDUP),
+
   /* Generation 4: CIS */
   DEVICE ("Canon PIXMA MP640", "MP640", MP640_PID, 4800, 638, 877, PIXMA_CAP_CIS),
   DEVICE ("Canon PIXMA MP560", "MP560", MP560_PID, 2400, 638, 877, PIXMA_CAP_CIS),
