@@ -1,5 +1,6 @@
 /* SANE - Scanner Access Now Easy.
 
+   Copyright (C) 2007-2008 Nicolas Martin, <nicols-guest at alioth dot debian dot org>
    Copyright (C) 2006-2007 Wittawat Yamwong <wittawat@web.de>
 
    This file is part of the SANE package.
@@ -42,8 +43,10 @@
  */
 #include "../include/sane/config.h"
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>		/* localtime(C90) */
 
 #include "pixma_rename.h"
 #include "pixma_common.h"
@@ -59,12 +62,23 @@
 #define IMAGE_BLOCK_SIZE (0xc000)
 #define CMDBUF_SIZE 512
 
+#define MP730_PID 0x262f
+#define MP700_PID 0x2630
+
 #define MP360_PID 0x263c
 #define MP370_PID 0x263d
 #define MP390_PID 0x263e
-#define MP700_PID 0x2630
-#define MP730_PID 0x262f
+#define MP375R_PID 0x263f   /* untested */
 
+#define MP740_PID 0x264c	/* Untested */
+#define MP710_PID 0x264d
+
+#define MF5730_PID 0x265d	/* Untested */
+#define MF5750_PID 0x265e	/* Untested */
+#define MF5770_PID 0x265f
+#define MF3110_PID 0x2660
+
+#define IR1020_PID 0x26e6
 
 enum mp730_state_t
 {
@@ -79,15 +93,16 @@ enum mp730_cmd_t
 {
   cmd_start_session = 0xdb20,
   cmd_select_source = 0xdd20,
-  cmd_gamma = 0xee20,
-  cmd_scan_param = 0xde20,
-  cmd_status = 0xf320,
+  cmd_gamma         = 0xee20,
+  cmd_scan_param    = 0xde20,
+  cmd_status        = 0xf320,
   cmd_abort_session = 0xef20,
-  cmd_time = 0xeb80,
-  cmd_read_image = 0xd420,
+  cmd_time          = 0xeb80,
+  cmd_read_image    = 0xd420,
+  cmd_error_info    = 0xff20,
 
-  cmd_activate = 0xcf60,
-  cmd_calibrate = 0xe920
+  cmd_activate      = 0xcf60,
+  cmd_calibrate     = 0xe920
 };
 
 typedef struct mp730_t
@@ -167,7 +182,21 @@ select_source (pixma_t * s)
 {
   mp730_t *mp = (mp730_t *) s->subdriver;
   uint8_t *data = pixma_newcmd (&mp->cb, cmd_select_source, 10, 0);
-  data[0] = (s->param->source == PIXMA_SOURCE_ADF) ? 2 : 1;
+  switch (s->param->source)
+    {
+    case PIXMA_SOURCE_ADF:
+      data[0] = 2;
+      break;
+
+    case PIXMA_SOURCE_ADFDUP:
+      data[0] = 2;
+      data[5] = 3;
+      break;
+
+    default:
+      data[0] = 1;
+      break;
+    }
   return pixma_exec (s, &mp->cb);
 }
 
@@ -184,11 +213,23 @@ send_scan_param (pixma_t * s)
   pixma_set_be32 (s->param->y, data + 0x0c);
   pixma_set_be32 (mp->raw_width, data + 0x10);
   pixma_set_be32 (s->param->h, data + 0x14);
-  data[0x18] = (s->param->channels == 1) ? 0x04 : 0x08;
-  data[0x19] = s->param->channels * s->param->depth;	/* bits per pixel */
-  data[0x1f] = 0x7f;
-  data[0x20] = 0xff;
+
+  if (s->param->channels == 1)
+    {
+      if (s->param->depth == 1)
+        data[0x18] = 0x01;
+      else
+        data[0x18] = 0x04;
+    }
+  else
+    data[0x18] = 0x08;
+
+  data[0x19] = s->param->channels * s->param->depth;  /* bits per pixel, for lineart should be 0x01 */
+  data[0x1e] = (s->param->depth == 1) ? 0x80 : 0x00;  /* modify for lineart: 0x80 NEW */
+  data[0x1f] = (s->param->depth == 1) ? 0x80 : 0x7f;  /* modify for lineart: 0x80 */
+  data[0x20] = (s->param->depth == 1) ? 0x01 : 0xff;  /* modify for lineart: 0x01 */
   data[0x23] = 0x81;
+
   return pixma_exec (s, &mp->cb);
 }
 
@@ -243,10 +284,21 @@ read_image_block (pixma_t * s, uint8_t * header, uint8_t * data)
 static int
 send_time (pixma_t * s)
 {
-  /* TODO */
-  UNUSED (s);
-  PDBG (pixma_dbg (3, "send_time() is not yet implemented.\n"));
-  return 0;
+  /* Why does a scanner need a time? */
+  time_t now;
+  struct tm *t;
+  uint8_t *data;
+  mp730_t *mp = (mp730_t *) s->subdriver;
+
+  data = pixma_newcmd (&mp->cb, cmd_time, 20, 0);
+  pixma_get_time (&now, NULL);
+  t = localtime (&now);
+  snprintf ((char *) data, 16,
+	    "%02d/%02d/%02d %02d:%02d",
+	    t->tm_year % 100, t->tm_mon + 1, t->tm_mday,
+	    t->tm_hour, t->tm_min);
+  PDBG (pixma_dbg (3, "Sending time: '%s'\n", (char *) data));
+  return pixma_exec (s, &mp->cb);
 }
 
 static int
@@ -264,7 +316,13 @@ handle_interrupt (pixma_t * s, int timeout)
     {
     case MP360_PID:
     case MP370_PID:
+    case MP375R_PID:
     case MP390_PID:
+    case MF5730_PID:
+    case MF5750_PID:
+    case MF5770_PID:
+    case MF3110_PID:
+    case IR1020_PID:
       if (len != 16)
 	{
 	  PDBG (pixma_dbg
@@ -273,9 +331,9 @@ handle_interrupt (pixma_t * s, int timeout)
 	}
       if (buf[12] & 0x40)
 	query_status (s);
-      /* FIXME: following is unverified! */
       if (buf[10] & 0x40)
 	send_time (s);
+      /* FIXME: following is unverified! */
       if (buf[15] & 1)
 	s->events = PIXMA_EV_BUTTON2;	/* b/w scan */
       if (buf[15] & 2)
@@ -284,6 +342,8 @@ handle_interrupt (pixma_t * s, int timeout)
 
     case MP700_PID:
     case MP730_PID:
+    case MP710_PID:
+    case MP740_PID:
       if (len != 8)
 	{
 	  PDBG (pixma_dbg
@@ -306,24 +366,92 @@ handle_interrupt (pixma_t * s, int timeout)
 static int
 has_ccd_sensor (pixma_t * s)
 {
-  return (s->cfg->pid == MP360_PID || s->cfg->pid == MP370_PID
-	  || s->cfg->pid == MP390_PID);
+  return (s->cfg->pid == MP360_PID ||
+          s->cfg->pid == MP370_PID ||
+          s->cfg->pid == MP375R_PID ||
+          s->cfg->pid == MP390_PID ||
+          s->cfg->pid == MF5730_PID ||
+          s->cfg->pid == MF5750_PID ||
+          s->cfg->pid == MF5770_PID);
+}
+
+static int
+read_error_info (pixma_t * s, void *buf, unsigned size)
+{
+  unsigned len = 16;
+  mp730_t *mp = (mp730_t *) s->subdriver;
+  uint8_t *data;
+  int error;
+
+  data = pixma_newcmd (&mp->cb, cmd_error_info, 0, len);
+  error = pixma_exec (s, &mp->cb);
+  if (error < 0)
+    return error;
+  if (buf && len < size)
+    {
+      size = len;
+      /* NOTE: I've absolutely no idea what the returned data mean. */
+      memcpy (buf, data, size);
+      error = len;
+    }
+  return error;
 }
 
 static int
 step1 (pixma_t * s)
 {
-  int error;
+  int error, tmo;
 
   error = query_status (s);
   if (error < 0)
     return error;
-  if (s->param->source == PIXMA_SOURCE_ADF && !has_paper (s))
+  if ((s->param->source == PIXMA_SOURCE_ADF
+       || s->param->source == PIXMA_SOURCE_ADFDUP)
+      && !has_paper (s))
     return PIXMA_ENO_PAPER;
   if (has_ccd_sensor (s))
     {
+      switch (s->cfg->pid)
+        {
+          case MF5730_PID:
+          case MF5750_PID:
+          case MF5770_PID:
+          /* MF57x0: Wait 10 sec before starting for 1st page only */
+            if (s->param->adf_pageid == 0)
+	      {
+                tmo = 10;  /* like Windows driver, 10 sec CCD calibration ? */
+                while (--tmo >= 0)
+                  {
+                    error = handle_interrupt (s, 1000);		\
+                    if (s->cancel)				\
+                      return PIXMA_ECANCELED;			\
+                    if (error != PIXMA_ECANCELED && error < 0)	\
+                      return error;
+                    PDBG (pixma_dbg (2, "CCD Calibration ends in %d sec.\n", tmo));
+                  }
+              }
+            break;
+
+          default:
+            break;
+        }
+        
       activate (s, 0);
       error = calibrate (s);
+      
+      switch (s->cfg->pid)
+        {
+          case MF5730_PID:
+          case MF5750_PID:
+          case MF5770_PID:
+          /* MF57x0: calibration returns PIXMA_STATUS_FAILED */
+            if (error == PIXMA_ECANCELED)
+              error = read_error_info (s, NULL, 0);
+            break;
+
+          default:
+            break;
+        }
     }
   if (error >= 0)
     error = activate (s, 0);
@@ -393,29 +521,72 @@ mp730_close (pixma_t * s)
 
   mp730_finish_scan (s);
   free (mp->cb.buf);
+  free (mp->buf);
   free (mp);
   s->subdriver = NULL;
 }
 
 static unsigned
-calc_raw_width (const pixma_scan_param_t * sp)
+calc_raw_width (pixma_t * s, const pixma_scan_param_t * sp)
 {
   unsigned raw_width;
   /* FIXME: Does MP730 need the alignment? */
+  /*  TODO test: MP710/740 */
   if (sp->channels == 1)
-    raw_width = ALIGN (sp->w, 12);
+    {
+      if (sp->depth == 8)   /* grayscale  */
+        {
+          if (s->cfg->pid == MP700_PID ||
+              s->cfg->pid == MP730_PID ||
+              s->cfg->pid == MP360_PID ||
+              s->cfg->pid == MP370_PID ||
+              s->cfg->pid == MP375R_PID ||
+              s->cfg->pid == MP390_PID ||
+	      s->cfg->pid == IR1020_PID)
+            raw_width = ALIGN_SUP (sp->w, 4);
+          else
+            raw_width = ALIGN_SUP (sp->w, 12);
+        }
+      else   /* depth = 1 : LINEART */
+        raw_width = ALIGN_SUP (sp->w, 16);
+    }
   else
-    raw_width = ALIGN (sp->w, 4);
+    raw_width = ALIGN_SUP (sp->w, 4);
   return raw_width;
 }
 
 static int
 mp730_check_param (pixma_t * s, pixma_scan_param_t * sp)
 {
-  UNUSED (s);
+  uint8_t k = 1;
+
+  /* check if channels is 3, or if depth is 1 then channels also 1 else set depth to 8 */
+  if ((sp->channels==3) || !(sp->channels==1 && sp->depth==1))
+    {
+      sp->depth=8;
+    }
+  /* for MP360/370, MP700/730 in grayscale & lineart modes, max scan res is 600 dpi */
+  if (s->cfg->pid == MP700_PID ||
+      s->cfg->pid == MP730_PID ||
+      s->cfg->pid == MP360_PID ||
+      s->cfg->pid == MP370_PID ||
+      s->cfg->pid == MP375R_PID ||
+      s->cfg->pid == MP390_PID)
+    {
+      if (sp->channels == 1)
+          k = sp->xdpi / MIN (sp->xdpi, 600);
+    }
+
+  sp->x /= k;
+  sp->y /= k;
+  sp->h /= k;
+  sp->xdpi /= k;
+  sp->ydpi = sp->xdpi;
+
+  sp->w = calc_raw_width (s, sp);
+  sp->w /= k;
+  sp->line_size = (calc_raw_width (s, sp) * sp->channels * sp->depth) / 8;
 
-  sp->depth = 8;		/* FIXME: Does MP730 supports other depth? */
-  sp->line_size = calc_raw_width (sp) * sp->channels;
   return 0;
 }
 
@@ -434,7 +605,7 @@ mp730_scan (pixma_t * s)
     {
     }
 
-  mp->raw_width = calc_raw_width (s->param);
+  mp->raw_width = calc_raw_width (s, s->param);
   PDBG (pixma_dbg (3, "raw_width = %u\n", mp->raw_width));
 
   n = IMAGE_BLOCK_SIZE / s->param->line_size + 1;
@@ -478,12 +649,8 @@ mp730_fill_buffer (pixma_t * s, pixma_imagebuf_t * ib)
 	{
 	  if (s->cancel)
 	    return PIXMA_ECANCELED;
-	  if (mp->last_block)
-	    {
-	      /* end of image */
-	      mp->state = state_finished;
+	  if (mp->last_block)           /* end of image */
 	      return 0;
-	    }
 
 	  error = read_image_block (s, header, mp->imgbuf + mp->imgbuf_len);
 	  if (error < 0)
@@ -492,6 +659,10 @@ mp730_fill_buffer (pixma_t * s, pixma_imagebuf_t * ib)
 	  bytes_received = error;
 	  block_size = pixma_get_be16 (header + 4);
 	  mp->last_block = ((header[2] & 0x28) == 0x28);
+	  if (mp->last_block)
+	    {    /* end of image */
+	      mp->state = state_finished;
+	    }
 	  if ((header[2] & ~0x38) != 0)
 	    {
 	      PDBG (pixma_dbg (1, "WARNING: Unexpected result header\n"));
@@ -504,7 +675,7 @@ mp730_fill_buffer (pixma_t * s, pixma_imagebuf_t * ib)
 	      /* no image data at this moment. */
 	      /*pixma_sleep(100000); *//* FIXME: too short, too long? */
 	      handle_interrupt (s, 100);
-	    /*XXX*/}
+            }
 	}
       while (block_size == 0);
 
@@ -514,16 +685,20 @@ mp730_fill_buffer (pixma_t * s, pixma_imagebuf_t * ib)
       /* n = number of full lines (rows) we have in the buffer. */
       if (n != 0)
 	{
-	  if (s->param->channels != 1)
+	  if (s->param->channels != 1    &&
+	      s->cfg->pid != MF5730_PID  &&
+	      s->cfg->pid != MF5750_PID  &&
+	      s->cfg->pid != MF5770_PID  &&
+	      s->cfg->pid != MF3110_PID  &&
+	      s->cfg->pid != IR1020_PID)
 	    {
-	      /* color */
+	      /* color, and not an MF57x0 nor MF3110 */
 	      pack_rgb (mp->imgbuf, n, mp->raw_width, mp->lbuf);
 	    }
 	  else
-	    {
-	      /* grayscale */
-	      memcpy (mp->lbuf, mp->imgbuf, n * s->param->line_size);
-	    }
+             /* grayscale/lineart or MF57x0 or MF3110 */
+             memcpy (mp->lbuf, mp->imgbuf, n * s->param->line_size);
+
 	  block_size = n * s->param->line_size;
 	  mp->imgbuf_len -= block_size;
 	  memcpy (mp->imgbuf, mp->imgbuf + block_size, mp->imgbuf_len);
@@ -539,7 +714,7 @@ mp730_fill_buffer (pixma_t * s, pixma_imagebuf_t * ib)
 static void
 mp730_finish_scan (pixma_t * s)
 {
-  int error;
+  int error, aborted = 0;
   mp730_t *mp = (mp730_t *) s->subdriver;
 
   switch (mp->state)
@@ -549,6 +724,7 @@ mp730_finish_scan (pixma_t * s)
       /* fall through */
     case state_scanning:
     case state_warmup:
+      aborted = 1;
       error = abort_session (s);
       if (error < 0)
 	PDBG (pixma_dbg
@@ -559,7 +735,20 @@ mp730_finish_scan (pixma_t * s)
       query_status (s);
       query_status (s);
       activate (s, 0);
-      free (mp->buf);
+
+      if (! aborted && s->cfg->pid == IR1020_PID)
+	{
+	  error = abort_session (s);
+	  if (error < 0)
+	    {
+	      PDBG (pixma_dbg
+		    (1, "WARNING:abort_session() failed %s\n",
+		     pixma_strerror (error)));
+	      query_status (s);
+	      query_status (s);
+	      activate (s, 0);
+	    }
+	}
       mp->buf = mp->lbuf = mp->imgbuf = NULL;
       mp->state = state_idle;
       /* fall through */
@@ -603,21 +792,33 @@ static const pixma_scan_ops_t pixma_mp730_ops = {
   mp730_get_status
 };
 
-#define DEVICE(name, pid, dpi, w, h, cap) {     \
-        name,              /* name */		\
-	0x04a9, pid,       /* vid pid */	\
-	1,                 /* iface */		\
-	&pixma_mp730_ops,  /* ops */		\
-	dpi, dpi,          /* xdpi, ydpi */	\
-	w, h,              /* width, height */	\
-        PIXMA_CAP_GRAY|PIXMA_CAP_EVENTS|cap                      \
+#define DEVICE(name, model, pid, dpi, w, h, cap) {           \
+              name,              /* name */           \
+              model,             /* model */		      \
+              0x04a9, pid,       /* vid pid */	      \
+              1,                 /* iface */		      \
+              &pixma_mp730_ops,  /* ops */            \
+              dpi, dpi,          /* xdpi, ydpi */	    \
+              w, h,              /* width, height */	\
+        PIXMA_CAP_GRAY|PIXMA_CAP_EVENTS|cap           \
 }
 const pixma_config_t pixma_mp730_devices[] = {
 /* TODO: check area limits */
-  DEVICE ("Canon SmartBase MP360", MP360_PID, 1200, 636, 868, 0),
-  DEVICE ("Canon SmartBase MP370", MP370_PID, 1200, 636, 868, 0),
-  DEVICE ("Canon SmartBase MP390", MP390_PID, 1200, 636, 868, 0),
-  DEVICE ("Canon MultiPASS MP700", MP700_PID, 1200, 638, 877 /*1035 */ , 0),
-  DEVICE ("Canon MultiPASS MP730", MP730_PID, 1200, 637, 868, PIXMA_CAP_ADF),
-  DEVICE (NULL, 0, 0, 0, 0, 0)
+  DEVICE ("PIXMA MP360", "MP360", MP360_PID, 1200, 636, 868, PIXMA_CAP_LINEART),
+  DEVICE ("PIXMA MP370", "MP370", MP370_PID, 1200, 636, 868, PIXMA_CAP_LINEART),
+  DEVICE ("PIXMA MP375R", "MP375R", MP375R_PID, 1200, 636, 868, PIXMA_CAP_LINEART),
+  DEVICE ("PIXMA MP390", "MP390", MP390_PID, 1200, 636, 868, PIXMA_CAP_LINEART),
+  DEVICE ("PIXMA MP700", "MP700", MP700_PID, 1200, 638, 877 /*1035 */ , PIXMA_CAP_LINEART),
+  DEVICE ("PIXMA MP710", "MP710", MP710_PID, 1200, 637, 868, PIXMA_CAP_LINEART),
+  DEVICE ("PIXMA MP730", "MP730", MP730_PID, 1200, 637, 868, PIXMA_CAP_ADF | PIXMA_CAP_LINEART),
+  DEVICE ("PIXMA MP740", "MP740", MP740_PID, 1200, 637, 868, PIXMA_CAP_ADF | PIXMA_CAP_LINEART),
+  
+  DEVICE ("Canon imageCLASS MF5730", "MF5730", MF5730_PID, 1200, 636, 868, PIXMA_CAP_ADF),
+  DEVICE ("Canon imageCLASS MF5750", "MF5750", MF5750_PID, 1200, 636, 868, PIXMA_CAP_ADF),
+  DEVICE ("Canon imageCLASS MF5770", "MF5770", MF5770_PID, 1200, 636, 868, PIXMA_CAP_ADF),
+  DEVICE ("Canon imageCLASS MF3110", "MF3110", MF3110_PID, 600, 636, 868, 0),
+
+  DEVICE ("Canon iR 1020/1024/1025", "iR1020", IR1020_PID, 600, 636, 868, PIXMA_CAP_ADFDUP),
+
+  DEVICE (NULL, NULL, 0, 0, 0, 0, 0)
 };
