@@ -109,6 +109,7 @@ typedef struct mp750_t
   uint8_t *buf, *rawimg, *img;
   unsigned rawimg_left, imgbuf_len, last_block_size, imgbuf_ofs;
   int shifted_bytes;
+  int stripe_shift; /* for 2400dpi */
   unsigned last_block;
 
   unsigned monochrome:1;
@@ -262,14 +263,14 @@ request_image_block_ex (pixma_t * s, unsigned *size, uint8_t * info,
   if (error >= 0)
     {
       if (mp->cb.reslen == 6)
-	{
-	  *info = mp->cb.buf[2];
-	  *size = pixma_get_be16 (mp->cb.buf + 4);
-	}
+        {
+          *info = mp->cb.buf[2];
+          *size = pixma_get_be16 (mp->cb.buf + 4);
+        }
       else
-	{
-	  error = PIXMA_EPROTO;
-	}
+        {
+          error = PIXMA_EPROTO;
+        }
     }
   return error;
 }
@@ -299,10 +300,10 @@ read_image_block (pixma_t * s, uint8_t * data)
     {
       int error;
       if ((error = pixma_read (s->io, &temp, 0)) < 0)
-	{
-	  PDBG (pixma_dbg
-		(1, "WARNING:reading zero-length packet failed %d\n", error));
-	}
+        {
+          PDBG (pixma_dbg
+          (1, "WARNING:reading zero-length packet failed %d\n", error));
+        }
     }
   return count;
 }
@@ -320,7 +321,7 @@ read_error_info (pixma_t * s, void *buf, unsigned size)
   if (error >= 0 && buf)
     {
       if (len < size)
-	size = len;
+        size = len;
       /* NOTE: I've absolutely no idea what the returned data mean. */
       memcpy (buf, data, size);
       error = len;
@@ -386,7 +387,7 @@ step1 (pixma_t * s)
   if (s->param->source == PIXMA_SOURCE_ADF && !has_paper (s))
     return PIXMA_ENO_PAPER;
   error = activate_cs (s, 0);
-  /*SIM*/ if (error < 0)
+   /*SIM*/ if (error < 0)
     return error;
   error = activate_cs (s, 0x20);
   if (error < 0)
@@ -408,13 +409,17 @@ step1 (pixma_t * s)
 
 static void
 shift_rgb (const uint8_t * src, unsigned pixels,
-	   int sr, int sg, int sb, uint8_t * dst)
+           int sr, int sg, int sb, int stripe_shift,
+           int line_size, uint8_t * dst)
 {
+  unsigned st;
+
   for (; pixels != 0; pixels--)
     {
-      *(dst++ + sr) = *src++;
-      *(dst++ + sg) = *src++;
-      *(dst++ + sb) = *src++;
+      st = (pixels % 2 == 0) ? -2 * stripe_shift * line_size : 0;
+      *(dst++ + sr + st) = *src++;
+      *(dst++ + sg + st) = *src++;
+      *(dst++ + sb + st) = *src++;
     }
 }
 
@@ -549,9 +554,9 @@ mp750_check_param (pixma_t * s, pixma_scan_param_t * sp)
 
   sp->depth = 8;		/* FIXME: Does MP750 supports other depth? */
   if (sp->channels == 1)
-    raw_width = ALIGN (sp->w, 12);
+    raw_width = ALIGN_SUP (sp->w, 12);
   else
-    raw_width = ALIGN (sp->w, 4);
+    raw_width = ALIGN_SUP (sp->w, 4);
   sp->line_size = raw_width * sp->channels;
   return 0;
 }
@@ -573,12 +578,17 @@ mp750_scan (pixma_t * s)
     }
 
   if (s->param->channels == 1)
-    mp->raw_width = ALIGN (s->param->w, 12);
+    mp->raw_width = ALIGN_SUP (s->param->w, 12);
   else
-    mp->raw_width = ALIGN (s->param->w, 4);
+    mp->raw_width = ALIGN_SUP (s->param->w, 4);
 
   dpi = s->param->ydpi;
-  spare = 2 * calc_component_shifting (s);	/* FIXME: or maybe (2*... + 1)? */
+
+  /* add a stripe shift for 2400dpi */
+  mp->stripe_shift = (dpi == 2400) ? 4 : 0;
+
+  /* modify for stripe shift */
+  spare = 2 * calc_component_shifting (s) + 2 * mp->stripe_shift; /* FIXME: or maybe (2*... + 1)? */
   mp->raw_height = s->param->h + spare;
   PDBG (pixma_dbg (3, "raw_width=%u raw_height=%u dpi=%u\n",
 		   mp->raw_width, mp->raw_height, dpi));
@@ -628,34 +638,34 @@ mp750_fill_buffer (pixma_t * s, pixma_imagebuf_t * ib)
 
       query_status (s);
       check_status (s);
-      /*SIM*/ while (!is_calibrated (s) && --tmo >= 0)
-	{
-	  if (s->cancel)
-	    return PIXMA_ECANCELED;
-	  if (handle_interrupt (s, 1000) > 0)
-	    {
-	      block_size = 0;
-	      error = request_image_block (s, &block_size, &info);
-	      /*SIM*/ if (error < 0)
-		return error;
-	    }
-	}
+ /*SIM*/ while (!is_calibrated (s) && --tmo >= 0)
+        {
+          if (s->cancel)
+            return PIXMA_ECANCELED;
+          if (handle_interrupt (s, 1000) > 0)
+            {
+              block_size = 0;
+              error = request_image_block (s, &block_size, &info);
+               /*SIM*/ if (error < 0)
+          return error;
+            }
+        }
       if (tmo < 0)
-	{
-	  PDBG (pixma_dbg (1, "WARNING:Timed out waiting for calibration\n"));
-	  return PIXMA_ETIMEDOUT;
-	}
+        {
+          PDBG (pixma_dbg (1, "WARNING:Timed out waiting for calibration\n"));
+          return PIXMA_ETIMEDOUT;
+        }
       pixma_sleep (100000);
       query_status (s);
       if (is_warming_up (s) || !is_calibrated (s))
-	{
-	  PDBG (pixma_dbg (1, "WARNING:Wrong status: wup=%d cal=%d\n",
-			   is_warming_up (s), is_calibrated (s)));
-	  return PIXMA_EPROTO;
-	}
+        {
+          PDBG (pixma_dbg (1, "WARNING:Wrong status: wup=%d cal=%d\n",
+               is_warming_up (s), is_calibrated (s)));
+          return PIXMA_EPROTO;
+        }
       block_size = 0;
       request_image_block (s, &block_size, &info);
-      /*SIM*/ mp->state = state_scanning;
+       /*SIM*/ mp->state = state_scanning;
       mp->last_block = 0;
     }
 
@@ -677,59 +687,59 @@ mp750_fill_buffer (pixma_t * s, pixma_imagebuf_t * ib)
   do
     {
       if (mp->last_block_size > 0)
-	{
-	  block_size = mp->imgbuf_len - mp->last_block_size;
-	  memcpy (mp->img, mp->img + mp->last_block_size, block_size);
-	}
+        {
+          block_size = mp->imgbuf_len - mp->last_block_size;
+          memcpy (mp->img, mp->img + mp->last_block_size, block_size);
+        }
 
       do
-	{
-	  if (s->cancel)
-	    return PIXMA_ECANCELED;
-	  if (mp->last_block)
-	    {
-	      /* end of image */
-	      info = mp->last_block;
-	      if (info != 0x38)
-		{
-		  query_status (s);
-		  /*SIM*/ while ((info & 0x28) != 0x28)
-		    {
-		      pixma_sleep (10000);
-		      error = request_image_block2 (s, &info);
-		      if (s->cancel)
-			return PIXMA_ECANCELED;	/* FIXME: Is it safe to cancel here? */
-		      if (error < 0)
-			return error;
-		    }
-		}
-	      mp->needs_abort = (info != 0x38);
-	      mp->last_block = info;
-	      mp->state = state_finished;
-	      return 0;
-	    }
+        {
+          if (s->cancel)
+            return PIXMA_ECANCELED;
+          if (mp->last_block)
+            {
+              /* end of image */
+              info = mp->last_block;
+              if (info != 0x38)
+                {
+                  query_status (s);
+                   /*SIM*/ while ((info & 0x28) != 0x28)
+                    {
+                      pixma_sleep (10000);
+                      error = request_image_block2 (s, &info);
+                      if (s->cancel)
+                        return PIXMA_ECANCELED;	/* FIXME: Is it safe to cancel here? */
+                      if (error < 0)
+                        return error;
+                    }
+                }
+              mp->needs_abort = (info != 0x38);
+              mp->last_block = info;
+              mp->state = state_finished;
+              return 0;
+            }
 
-	  check_status (s);
-	  /*SIM*/ while (handle_interrupt (s, 1) > 0);
-	  /*SIM*/ block_size = IMAGE_BLOCK_SIZE;
-	  error = request_image_block (s, &block_size, &info);
-	  if (error < 0)
-	    {
-	      if (error == PIXMA_ECANCELED)
-		read_error_info (s, NULL, 0);
-	      return error;
-	    }
-	  mp->last_block = info;
-	  if ((info & ~0x38) != 0)
-	    {
-	      PDBG (pixma_dbg (1, "WARNING: Unknown info byte %x\n", info));
-	    }
-	  if (block_size == 0)
-	    {
-	      /* no image data at this moment. */
-	      pixma_sleep (10000);
-	    }
-	}
+          check_status (s);
+           /*SIM*/ while (handle_interrupt (s, 1) > 0);
+           /*SIM*/ block_size = IMAGE_BLOCK_SIZE;
+          error = request_image_block (s, &block_size, &info);
+          if (error < 0)
+            {
+              if (error == PIXMA_ECANCELED)
+                read_error_info (s, NULL, 0);
+              return error;
+            }
+          mp->last_block = info;
+          if ((info & ~0x38) != 0)
+            {
+              PDBG (pixma_dbg (1, "WARNING: Unknown info byte %x\n", info));
+            }
+          if (block_size == 0)
+            {
+              /* no image data at this moment. */
+              pixma_sleep (10000);
+            }
+        }
       while (block_size == 0);
 
       error = read_image_block (s, mp->rawimg + mp->rawimg_left);
@@ -746,7 +756,8 @@ mp750_fill_buffer (pixma_t * s, pixma_imagebuf_t * ib)
       n = mp->rawimg_left / 3;
       /* n = number of pixels in the buffer */
       shift_rgb (mp->rawimg, n, shift[0], shift[1], shift[2],
-		 mp->img + mp->imgbuf_ofs);
+                 mp->stripe_shift, s->param->line_size,
+                 mp->img + mp->imgbuf_ofs);
       n *= 3;
       mp->shifted_bytes += n;
       mp->rawimg_left -= n;	/* rawimg_left = 0, 1 or 2 bytes left in the buffer. */
@@ -791,7 +802,7 @@ mp750_finish_scan (pixma_t * s)
 	    }
 	}
       query_status (s);
-      /*SIM*/ activate (s, 0);
+       /*SIM*/ activate (s, 0);
       if (mp->needs_abort)
 	{
 	  mp->needs_abort = 0;
@@ -844,8 +855,9 @@ static const pixma_scan_ops_t pixma_mp750_ops = {
   mp750_get_status
 };
 
-#define DEVICE(name, pid, dpi, cap) {			\
+#define DEVICE(name, model, pid, dpi, cap) {			\
 	name,                  /* name */		\
+	model,                 /* model */		\
 	0x04a9, pid,           /* vid pid */		\
 	0,                     /* iface */		\
 	&pixma_mp750_ops,      /* ops */		\
@@ -855,8 +867,8 @@ static const pixma_scan_ops_t pixma_mp750_ops = {
 }
 
 const pixma_config_t pixma_mp750_devices[] = {
-  DEVICE ("Canon PIXMA MP750", MP750_PID, 2400, PIXMA_CAP_ADF),
-  DEVICE ("Canon PIXMA MP760", MP760_PID, 2400, PIXMA_CAP_TPU),
-  DEVICE ("Canon PIXMA MP780", MP780_PID, 2400, PIXMA_CAP_ADF),
-  DEVICE (NULL, 0, 0, 0)
+  DEVICE ("Canon PIXMA MP750", "MP750", MP750_PID, 2400, PIXMA_CAP_ADF),
+  DEVICE ("Canon PIXMA MP760", "MP760", MP760_PID, 2400, PIXMA_CAP_TPU),
+  DEVICE ("Canon PIXMA MP780", "MP780", MP780_PID, 2400, PIXMA_CAP_ADF),
+  DEVICE (NULL, NULL, 0, 0, 0)
 };
