@@ -20,7 +20,7 @@
 
    This file implements a SANE backend for Siemens 9036 flatbed scanners.  */
 
-#include "sane/config.h"
+#include "../include/sane/config.h"
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -31,17 +31,16 @@
 # define PATH_MAX	1024
 #endif
 
-#include "sane/sane.h"
-#include "sane/sanei.h"
-#include "sane/sanei_config.h"
-#include "sane/saneopts.h"
-#include "sane/sanei_scsi.h"
+#include "../include/sane/sane.h"
+#include "../include/sane/sanei.h"
+#include "../include/sane/sanei_config.h"
+#include "../include/sane/saneopts.h"
+#include "../include/sane/sanei_scsi.h"
 #include "s9036.h"
 
 #define BACKEND_NAME	s9036
-#include "sane/sanei_backend.h"
+#include "../include/sane/sanei_backend.h"
 
-#define MM_PER_INCH	25.4
 
 #undef Byte
 #define Byte SANE_Byte
@@ -834,7 +833,7 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   DBG_INIT ();
 
   if (version_code)
-    *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
+    *version_code = SANE_VERSION_CODE (SANE_CURRENT_MAJOR, V_MINOR, 0);
 
   fp = sanei_config_open ("s9036.conf");
   if (!fp)
